/**
 * Copyright (c) 2021 OceanBase
 * OceanBase CE is licensed under Mulan PubL v2.
 * You can use this software according to the terms and conditions of the Mulan PubL v2.
 * You may obtain a copy of Mulan PubL v2 at:
 *          http://license.coscl.org.cn/MulanPubL-2.0
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PubL v2 for more details.
 */

#ifndef OCEANBASE_SQL_EXPR_FUNC_REVERSE_
#define OCEANBASE_SQL_EXPR_FUNC_REVERSE_

#include "sql/engine/expr/ob_expr_operator.h"

namespace oceanbase {
namespace sql {
class ObExprReverse : public ObStringExprOperator {
public:
  explicit ObExprReverse(common::ObIAllocator& alloc);
  virtual ~ObExprReverse();
  virtual int calc_result_type1(ObExprResType& type, ObExprResType& type1, common::ObExprTypeCtx& type_ctx) const;
  virtual int calc_result1(common::ObObj& result, const common::ObObj& input, common::ObExprCtx& expr_ctx) const;
  virtual int cg_expr(ObExprCGCtx& expr_cg_ctx, const ObRawExpr& raw_expr, ObExpr& rt_expr) const;
  static int do_reverse(const common::ObString& input_str, const common::ObCollationType& cs_type,
      common::ObIAllocator* allocator, common::ObString& res_str);

private:
  DISALLOW_COPY_AND_ASSIGN(ObExprReverse);
};

inline int ObExprReverse::calc_result_type1(
    ObExprResType& type, ObExprResType& type1, common::ObExprTypeCtx& type_ctx) const
{
  UNUSED(type_ctx);
  type1.set_calc_type(common::ObVarcharType);
  type1.set_calc_collation_type(type1.get_collation_type());
  type.set_varchar();
  type.set_length(type1.get_length());
  return aggregate_charsets_for_string_result(type, &type1, 1, type_ctx.get_coll_type());
}

}  // namespace sql
}  // namespace oceanbase

#endif /* OCEANBASE_SQL_EXPR_FUNC_REVERSE_ */
