/**
 * Copyright (c) 2021 OceanBase
 * OceanBase CE is licensed under Mulan PubL v2.
 * You can use this software according to the terms and conditions of the Mulan PubL v2.
 * You may obtain a copy of Mulan PubL v2 at:
 *          http://license.coscl.org.cn/MulanPubL-2.0
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PubL v2 for more details.
 */

#ifndef _OCEANBASE_SQL_OB_EXPR_TO_BLOB_H_
#define _OCEANBASE_SQL_OB_EXPR_TO_BLOB_H_
#include "sql/engine/expr/ob_expr_operator.h"
#include "share/object/ob_obj_cast.h"

namespace oceanbase {
namespace sql {

// https://docs.oracle.com/cd/B28359_01/server.111/b28286/functions186.htm#SQLRF30029
class ObExprToBlob : public ObStringExprOperator {
public:
  explicit ObExprToBlob(common::ObIAllocator& alloc);
  virtual ~ObExprToBlob();
  virtual int calc_result_type1(ObExprResType& type, ObExprResType& text, common::ObExprTypeCtx& type_ctx) const;
  virtual int calc_result1(common::ObObj& result, const common::ObObj& text, common::ObExprCtx& expr_ctx) const;
  virtual int cg_expr(ObExprCGCtx& expr_cg_ctx, const ObRawExpr& raw_expr, ObExpr& rt_expr) const;
  static int eval_to_blob(const ObExpr& expr, ObEvalCtx& ctx, ObDatum& res);

private:
  DISALLOW_COPY_AND_ASSIGN(ObExprToBlob);
};

}  // namespace sql
}  // namespace oceanbase

#endif  // OB_EXPR_TO_BLOB_H_
