/**
 * Copyright (c) 2021 OceanBase
 * OceanBase CE is licensed under Mulan PubL v2.
 * You can use this software according to the terms and conditions of the Mulan PubL v2.
 * You may obtain a copy of Mulan PubL v2 at:
 *          http://license.coscl.org.cn/MulanPubL-2.0
 * THIS SOFTWARE IS PROVIDED ON AN "AS IS" BASIS, WITHOUT WARRANTIES OF ANY KIND,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO NON-INFRINGEMENT,
 * MERCHANTABILITY OR FIT FOR A PARTICULAR PURPOSE.
 * See the Mulan PubL v2 for more details.
 */
#include "sql/parser/ob_item_type.h"
const char* get_type_name(int type)
{
	switch(type){
	case T_NULL : return "T_NULL";
	case T_TINYINT : return "T_TINYINT";
	case T_SMALLINT : return "T_SMALLINT";
	case T_MEDIUMINT : return "T_MEDIUMINT";
	case T_INT32 : return "T_INT32";
	case T_INT : return "T_INT";
	case T_UTINYINT : return "T_UTINYINT";
	case T_USMALLINT : return "T_USMALLINT";
	case T_UMEDIUMINT : return "T_UMEDIUMINT";
	case T_UINT32 : return "T_UINT32";
	case T_UINT64 : return "T_UINT64";
	case T_FLOAT : return "T_FLOAT";
	case T_DOUBLE : return "T_DOUBLE";
	case T_UFLOAT : return "T_UFLOAT";
	case T_UDOUBLE : return "T_UDOUBLE";
	case T_NUMBER : return "T_NUMBER";
	case T_UNUMBER : return "T_UNUMBER";
	case T_DATETIME : return "T_DATETIME";
	case T_TIMESTAMP : return "T_TIMESTAMP";
	case T_DATE : return "T_DATE";
	case T_TIME : return "T_TIME";
	case T_YEAR : return "T_YEAR";
	case T_VARCHAR : return "T_VARCHAR"; /*ObVarcharType*/
	case T_CHAR : return "T_CHAR";
	case T_HEX_STRING : return "T_HEX_STRING";
	case T_EXTEND : return "T_EXTEND";
	case T_QUESTIONMARK : return "T_QUESTIONMARK"; /*ObUnknownType*/
	case T_TINYTEXT : return "T_TINYTEXT";
	case T_TEXT : return "T_TEXT";
	case T_MEDIUMTEXT : return "T_MEDIUMTEXT";
	case T_LONGTEXT : return "T_LONGTEXT";
	case T_BIT : return "T_BIT";
	case T_ENUM : return "T_ENUM";
	case T_SET : return "T_SET";
	case T_TIMESTAMP_TZ : return "T_TIMESTAMP_TZ";
	case T_TIMESTAMP_LTZ : return "T_TIMESTAMP_LTZ";
	case T_TIMESTAMP_NANO : return "T_TIMESTAMP_NANO";
	case T_RAW : return "T_RAW";
	case T_INTERVAL_YM : return "T_INTERVAL_YM";
	case T_INTERVAL_DS : return "T_INTERVAL_DS";
	case T_NUMBER_FLOAT : return "T_NUMBER_FLOAT";
	case T_NVARCHAR2 : return "T_NVARCHAR2";
	case T_NCHAR : return "T_NCHAR";
	case T_UROWID : return "T_UROWID";
	case T_LOB : return "T_LOB";
	case T_BOOL : return "T_BOOL"; /*@todo remove later*/
	case T_MAX_CONST : return "T_MAX_CONST";
	case T_VAR_INT : return "T_VAR_INT";
	case T_EXEC_VAR : return "T_EXEC_VAR";
	case T_REF_ALIAS_COLUMN : return "T_REF_ALIAS_COLUMN";
	case T_REF_COLUMN : return "T_REF_COLUMN";
	case T_FUN_MAX : return "T_FUN_MAX";
	case T_FUN_MIN : return "T_FUN_MIN";
	case T_FUN_SUM : return "T_FUN_SUM";
	case T_FUN_COUNT_SUM : return "T_FUN_COUNT_SUM";
	case T_FUN_COUNT : return "T_FUN_COUNT";
	case T_FUN_GROUP_CONCAT : return "T_FUN_GROUP_CONCAT";
	case T_FUN_AVG : return "T_FUN_AVG";
	case T_FUN_APPROX_COUNT_DISTINCT : return "T_FUN_APPROX_COUNT_DISTINCT";
	case T_FUN_APPROX_COUNT_DISTINCT_SYNOPSIS : return "T_FUN_APPROX_COUNT_DISTINCT_SYNOPSIS";
	case T_FUN_APPROX_COUNT_DISTINCT_SYNOPSIS_MERGE : return "T_FUN_APPROX_COUNT_DISTINCT_SYNOPSIS_MERGE";
	case T_MIN_OP : return "T_MIN_OP";  /* 1. arithmetic operators */
	case T_OP_NEG : return "T_OP_NEG";  // negative
	case T_OP_POS : return "T_OP_POS";  // positive
	case T_OP_ABS : return "T_OP_ABS";  // for abs
	case T_OP_ADD : return "T_OP_ADD";
	case T_OP_MINUS : return "T_OP_MINUS";
	case T_OP_MUL : return "T_OP_MUL";
	case T_OP_DIV : return "T_OP_DIV";
	case T_OP_REM : return "T_OP_REM";  // remainder
	case T_OP_MOD : return "T_OP_MOD";
	case T_OP_EQ : return "T_OP_EQ"; /* 2. Bool operators */
	case T_OP_NSEQ : return "T_OP_NSEQ";
	case T_OP_LE : return "T_OP_LE";
	case T_OP_LT : return "T_OP_LT";
	case T_OP_GE : return "T_OP_GE";
	case T_OP_GT : return "T_OP_GT";
	case T_OP_NE : return "T_OP_NE";
	case T_OP_SQ_EQ : return "T_OP_SQ_EQ";
	case T_OP_SQ_NSEQ : return "T_OP_SQ_NSEQ";
	case T_OP_SQ_LE : return "T_OP_SQ_LE";
	case T_OP_SQ_LT : return "T_OP_SQ_LT";
	case T_OP_SQ_GE : return "T_OP_SQ_GE";
	case T_OP_SQ_GT : return "T_OP_SQ_GT";
	case T_OP_SQ_NE : return "T_OP_SQ_NE";
	case T_OP_POW : return "T_OP_POW";
	case T_OP_IS : return "T_OP_IS";
	case T_OP_IS_NOT : return "T_OP_IS_NOT";
	case T_OP_BTW : return "T_OP_BTW";
	case T_OP_NOT_BTW : return "T_OP_NOT_BTW";
	case T_OP_LIKE : return "T_OP_LIKE";
	case T_OP_NOT_LIKE : return "T_OP_NOT_LIKE";
	case T_OP_REGEXP : return "T_OP_REGEXP";
	case T_OP_NOT_REGEXP : return "T_OP_NOT_REGEXP";
	case T_OP_NOT : return "T_OP_NOT";
	case T_OP_AND : return "T_OP_AND";
	case T_OP_OR : return "T_OP_OR";
	case T_OP_IN : return "T_OP_IN";
	case T_OP_NOT_IN : return "T_OP_NOT_IN";
	case T_OP_ARG_CASE : return "T_OP_ARG_CASE";
	case T_OP_CASE : return "T_OP_CASE";
	case T_OP_ROW : return "T_OP_ROW";
	case T_OP_EXISTS : return "T_OP_EXISTS";
	case T_OP_NOT_EXISTS : return "T_OP_NOT_EXISTS";
	case T_OP_QUOTE : return "T_OP_QUOTE";
	case T_OP_CONV : return "T_OP_CONV";
	case T_OP_CNN : return "T_OP_CNN"; /* 3. String operators */
	case T_OP_INT_DIV : return "T_OP_INT_DIV";
	case T_OP_SIGN : return "T_OP_SIGN";
	case T_OP_ASSIGN : return "T_OP_ASSIGN";
	case T_OP_GET_USER_VAR : return "T_OP_GET_USER_VAR";
	case T_OP_GET_SYS_VAR : return "T_OP_GET_SYS_VAR";
	case T_OP_COLUMN_REF : return "T_OP_COLUMN_REF"; /* normalize column to function */
	case T_OP_CONST_VAL : return "T_OP_CONST_VAL";  /* normalize const to funciton */
	case T_OP_SET : return "T_OP_SET";
	case T_OP_UNION : return "T_OP_UNION";
	case T_OP_INTERSECT : return "T_OP_INTERSECT";
	case T_OP_EXCEPT : return "T_OP_EXCEPT";
	case T_OP_BIT_AND : return "T_OP_BIT_AND";
	case T_OP_BIT_OR : return "T_OP_BIT_OR";
	case T_OP_BIT_XOR : return "T_OP_BIT_XOR";
	case T_OP_BIT_INVERT : return "T_OP_BIT_INVERT";
	case T_OP_BIT_NEG : return "T_OP_BIT_NEG";
	case T_OP_BIT_LEFT_SHIFT : return "T_OP_BIT_LEFT_SHIFT";
	case T_OP_BIT_RIGHT_SHIFT : return "T_OP_BIT_RIGHT_SHIFT";
	case T_OP_AGG_PARAM_LIST : return "T_OP_AGG_PARAM_LIST";
	case T_REF_QUERY : return "T_REF_QUERY";
	case T_OP_PRIOR : return "T_OP_PRIOR";
	case T_OBJ_ACCESS_REF : return "T_OBJ_ACCESS_REF";
	case T_OP_CONNECT_BY_ROOT : return "T_OP_CONNECT_BY_ROOT";
	case T_OP_REGEXP_SUBSTR : return "T_OP_REGEXP_SUBSTR";
	case T_OP_GET_PACKAGE_VAR : return "T_OP_GET_PACKAGE_VAR";
	case T_OP_SHADOW_UK_PROJECT : return "T_OP_SHADOW_UK_PROJECT";
	case T_OP_XOR : return "T_OP_XOR";
	case T_OP_ORACLE_OUTER_JOIN_SYMBOL : return "T_OP_ORACLE_OUTER_JOIN_SYMBOL";
	case T_OP_RANGE_PARAM : return "T_OP_RANGE_PARAM";
	case T_OP_GET_SUBPROGRAM_VAR : return "T_OP_GET_SUBPROGRAM_VAR";
	case T_OP_MULTISET : return "T_OP_MULTISET";
	case T_OP_COLL_PRED : return "T_OP_COLL_PRED";  // collection predicate, such as: xx is member of collection
	case T_OP_BOOL : return "T_OP_BOOL";
	case T_OP_STACK_OVERFLOW_CHECK : return "T_OP_STACK_OVERFLOW_CHECK";  // stack over flow check expr.
	case T_OP_JOIN_BLOOM_FILTER : return "T_OP_JOIN_BLOOM_FILTER";
	case T_OP_TO_OUTFILE_ROW : return "T_OP_TO_OUTFILE_ROW";
	case T_OP_AGG_ADD : return "T_OP_AGG_ADD";
	case T_OP_AGG_MINUS : return "T_OP_AGG_MINUS";
	case T_OP_AGG_MUL : return "T_OP_AGG_MUL";
	case T_OP_AGG_DIV : return "T_OP_AGG_DIV";
	case T_FUN_SYS : return "T_FUN_SYS";
	case T_FUN_SYS_CAST : return "T_FUN_SYS_CAST";
	case T_FUN_SYS_CHAR : return "T_FUN_SYS_CHAR";
	case T_FUN_SYS_CONVERT : return "T_FUN_SYS_CONVERT";
	case T_FUN_SYS_GREATEST : return "T_FUN_SYS_GREATEST";
	case T_FUN_SYS_LEAST : return "T_FUN_SYS_LEAST";
	case T_FUN_SYS_COALESCE : return "T_FUN_SYS_COALESCE";
	case T_FUN_SYS_TRIM : return "T_FUN_SYS_TRIM";
	case T_FUN_SYS_SUBSTR : return "T_FUN_SYS_SUBSTR";
	case T_FUN_SYS_SUBSTRING_INDEX : return "T_FUN_SYS_SUBSTRING_INDEX";
	case T_FUN_SYS_MD5 : return "T_FUN_SYS_MD5";
	case T_FUN_SYS_IP2INT : return "T_FUN_SYS_IP2INT";
	case T_FUN_SYS_INT2IP : return "T_FUN_SYS_INT2IP";
	case T_FUN_SYS_INSERT : return "T_FUN_SYS_INSERT";
	case T_FUN_SYS_LENGTH : return "T_FUN_SYS_LENGTH";
	case T_FUN_SYS_HEX : return "T_FUN_SYS_HEX";
	case T_FUN_SYS_UNHEX : return "T_FUN_SYS_UNHEX";
	case T_FUN_SYS_UPPER : return "T_FUN_SYS_UPPER";
	case T_FUN_SYS_LOWER : return "T_FUN_SYS_LOWER";
	case T_FUN_SYS_BIN : return "T_FUN_SYS_BIN";
	case T_FUN_SYS_DATE : return "T_FUN_SYS_DATE";
	case T_FUN_SYS_DATE_ADD : return "T_FUN_SYS_DATE_ADD";
	case T_FUN_SYS_DATE_SUB : return "T_FUN_SYS_DATE_SUB";
	case T_FUN_SYS_DATE_DIFF : return "T_FUN_SYS_DATE_DIFF";
	case T_FUN_SYS_TIME_STAMP_DIFF : return "T_FUN_SYS_TIME_STAMP_DIFF";
	case T_FUN_SYS_TIME_DIFF : return "T_FUN_SYS_TIME_DIFF";
	case T_FUN_SYS_PERIOD_DIFF : return "T_FUN_SYS_PERIOD_DIFF";
	case T_FUN_SYS_FROM_UNIX_TIME : return "T_FUN_SYS_FROM_UNIX_TIME";
	case T_FUN_SYS_EXTRACT : return "T_FUN_SYS_EXTRACT";
	case T_FUN_SYS_MONTH : return "T_FUN_SYS_MONTH";
	case T_FUN_SYS_DATE_FORMAT : return "T_FUN_SYS_DATE_FORMAT";
	case T_FUN_SYS_STR_TO_DATE : return "T_FUN_SYS_STR_TO_DATE";
	case T_FUN_SYS_UTC_TIMESTAMP : return "T_FUN_SYS_UTC_TIMESTAMP";
	case T_FUN_SYS_UPS_TIME : return "T_FUN_SYS_UPS_TIME";
	case T_FUN_SYS_CUR_TIMESTAMP : return "T_FUN_SYS_CUR_TIMESTAMP";
	case T_FUN_SYS_CUR_TIME : return "T_FUN_SYS_CUR_TIME";
	case T_FUN_SYS_CUR_DATE : return "T_FUN_SYS_CUR_DATE";
	case T_FUN_SYS_TO_DAYS : return "T_FUN_SYS_TO_DAYS";
	case T_FUN_SYS_FROM_DAYS : return "T_FUN_SYS_FROM_DAYS";
	case T_FUN_SYS_NVL : return "T_FUN_SYS_NVL";
	case T_FUN_SYS_TIME_TO_USEC : return "T_FUN_SYS_TIME_TO_USEC";
	case T_FUN_SYS_USEC_TO_TIME : return "T_FUN_SYS_USEC_TO_TIME";
	case T_FUN_SYS_UNIX_TIMESTAMP : return "T_FUN_SYS_UNIX_TIMESTAMP";
	case T_FUN_SYS_ROUND : return "T_FUN_SYS_ROUND";
	case T_FUN_SYS_DUMP : return "T_FUN_SYS_DUMP";
	case T_FUN_SYS_FLOOR : return "T_FUN_SYS_FLOOR";
	case T_FUN_SYS_CEIL : return "T_FUN_SYS_CEIL";
	case T_FUN_SYS_REPEAT : return "T_FUN_SYS_REPEAT";
	case T_FUN_SYS_REPLACE : return "T_FUN_SYS_REPLACE";
	case T_FUN_SYS_MERGING_FROZEN_TIME : return "T_FUN_SYS_MERGING_FROZEN_TIME";
	case T_FUN_SYS_PART_HASH_V1 : return "T_FUN_SYS_PART_HASH_V1";
	case T_FUN_SYS_PART_KEY_V1 : return "T_FUN_SYS_PART_KEY_V1";
	case T_FUN_SYS_ADDR_TO_PART_ID : return "T_FUN_SYS_ADDR_TO_PART_ID";
	case T_FUN_SYS_DATABASE : return "T_FUN_SYS_DATABASE";
	case T_FUN_SYS_AUTOINC_NEXTVAL : return "T_FUN_SYS_AUTOINC_NEXTVAL";
	case T_FUN_SYS_LAST_INSERT_ID : return "T_FUN_SYS_LAST_INSERT_ID";
	case T_FUN_SYS_INSTR : return "T_FUN_SYS_INSTR";
	case T_FUN_SYS_LNNVL : return "T_FUN_SYS_LNNVL";
	case T_FUN_SYS_LOCATE : return "T_FUN_SYS_LOCATE";
	case T_FUN_SYS_EFFECTIVE_TENANT : return "T_FUN_SYS_EFFECTIVE_TENANT";
	case T_FUN_SYS_CURRENT_USER : return "T_FUN_SYS_CURRENT_USER";
	case T_FUN_SYS_USER : return "T_FUN_SYS_USER";
	case T_FUN_SYS_VERSION : return "T_FUN_SYS_VERSION";
	case T_FUN_SYS_CHARSET : return "T_FUN_SYS_CHARSET";
	case T_FUN_SYS_COLLATION : return "T_FUN_SYS_COLLATION";
	case T_FUN_SYS_SYS_VIEW_BIGINT_PARAM : return "T_FUN_SYS_SYS_VIEW_BIGINT_PARAM";
	case T_FUN_SYS_COERCIBILITY : return "T_FUN_SYS_COERCIBILITY";
	case T_FUN_SYS_REVERSE : return "T_FUN_SYS_REVERSE";
	case T_FUN_SYS_RIGHT : return "T_FUN_SYS_RIGHT";
	case T_FUN_SYS_EFFECTIVE_TENANT_ID : return "T_FUN_SYS_EFFECTIVE_TENANT_ID";
	case T_FUN_SYS_VALUES : return "T_FUN_SYS_VALUES";
	case T_FUN_SYS_OCT : return "T_FUN_SYS_OCT";
	case T_FUN_SYS_RPAD : return "T_FUN_SYS_RPAD";
	case T_FUN_SYS_SET_COLLATION : return "T_FUN_SYS_SET_COLLATION";
	case T_FUN_SYS_CHAR_LENGTH : return "T_FUN_SYS_CHAR_LENGTH";
	case T_FUN_SYS_CONNECTION_ID : return "T_FUN_SYS_CONNECTION_ID";
	case T_FUN_SYS_HOST_IP : return "T_FUN_SYS_HOST_IP";
	case T_FUN_SYS_RPC_PORT : return "T_FUN_SYS_RPC_PORT";
	case T_FUN_SYS_MYSQL_PORT : return "T_FUN_SYS_MYSQL_PORT";
	case T_FUN_SYS_IFNULL : return "T_FUN_SYS_IFNULL";
	case T_FUN_SYS_FOUND_ROWS : return "T_FUN_SYS_FOUND_ROWS";
	case T_FUN_SYS_ROW_COUNT : return "T_FUN_SYS_ROW_COUNT";
	case T_FUN_CNN_WS : return "T_FUN_CNN_WS";
	case T_FUN_SYS_SYS_PRIVILEGE_CHECK : return "T_FUN_SYS_SYS_PRIVILEGE_CHECK";
	case T_FUN_SYS_CMP_META : return "T_FUN_SYS_CMP_META";
	case T_FUN_SYS_DEFAULT : return "T_FUN_SYS_DEFAULT";
	case T_FUN_SYS_FIELD : return "T_FUN_SYS_FIELD";
	case T_FUN_SYS_NULLIF : return "T_FUN_SYS_NULLIF";
	case T_FUN_PAD : return "T_FUN_PAD";
	case T_FUN_COLUMN_CONV : return "T_FUN_COLUMN_CONV";
	case T_FUN_SYS_IF : return "T_FUN_SYS_IF";
	case T_FUN_SYS_ISNULL : return "T_FUN_SYS_ISNULL";
	case T_FUN_SYS_TIMESTAMP_NVL : return "T_FUN_SYS_TIMESTAMP_NVL";
	case T_FUN_IS_SERVING_TENANT : return "T_FUN_IS_SERVING_TENANT";
	case T_FUN_SYS_POSITION : return "T_FUN_SYS_POSITION";
	case T_FUN_SYS_ORA_DECODE : return "T_FUN_SYS_ORA_DECODE";
	case T_FUN_SYS_ELT : return "T_FUN_SYS_ELT";
	case T_FUN_SYS_STRCMP : return "T_FUN_SYS_STRCMP";
	case T_FUN_SYS_PART_KEY_V2 : return "T_FUN_SYS_PART_KEY_V2";
	case T_FUN_INNER_TRIM : return "T_FUN_INNER_TRIM";
	case T_FUN_SYS_PART_ID : return "T_FUN_SYS_PART_ID";
	case T_FUN_SYS_ORA_TRUNC : return "T_FUN_SYS_ORA_TRUNC";
	case T_FUN_SYS_TO_TYPE : return "T_FUN_SYS_TO_TYPE";
	case T_FUN_SYS_SLEEP : return "T_FUN_SYS_SLEEP";
	case T_FUN_GET_LAST_EXEC_ID : return "T_FUN_GET_LAST_EXEC_ID";
	case T_FUN_GET_LAST_TRACE_ID : return "T_FUN_GET_LAST_TRACE_ID";
	case T_FUN_SYS_CEILING : return "T_FUN_SYS_CEILING";
	case T_FUN_DES_HEX_STR : return "T_FUN_DES_HEX_STR";
	case T_FUN_SYS_YEAR : return "T_FUN_SYS_YEAR";
	case T_FUN_SYS_UUID : return "T_FUN_SYS_UUID";
	case T_FUN_SYS_TIME : return "T_FUN_SYS_TIME";
	case T_FUN_SYS_ASCII : return "T_FUN_SYS_ASCII";
	case T_FUN_SYS_ORD : return "T_FUN_SYS_ORD";
	case T_FUN_SYS_WORD_SEGMENT : return "T_FUN_SYS_WORD_SEGMENT";
	case T_FUN_SYS_LTRIM : return "T_FUN_SYS_LTRIM";
	case T_FUN_SYS_SPACE : return "T_FUN_SYS_SPACE";
	case T_FUN_SYS_RTRIM : return "T_FUN_SYS_RTRIM";
	case T_FUN_SYS_LPAD : return "T_FUN_SYS_LPAD";
	case T_FUN_SYS_TIME_STAMP_ADD : return "T_FUN_SYS_TIME_STAMP_ADD";
	case T_FUN_SYS_BIT_COUNT : return "T_FUN_SYS_BIT_COUNT";
	case T_FUN_SYS_FIND_IN_SET : return "T_FUN_SYS_FIND_IN_SET";
	case T_FUN_SYS_LEFT : return "T_FUN_SYS_LEFT";
	case T_FUN_SYS_MAKE_SET : return "T_FUN_SYS_MAKE_SET";
	case T_FUN_SYS_RAND : return "T_FUN_SYS_RAND";
	case T_FUN_SYS_ROWNUM : return "T_FUN_SYS_ROWNUM";
	case T_FUN_SYS_ESTIMATE_NDV : return "T_FUN_SYS_ESTIMATE_NDV";
	case T_FUN_ENUM_TO_STR : return "T_FUN_ENUM_TO_STR";
	case T_FUN_ENUM_TO_INNER_TYPE : return "T_FUN_ENUM_TO_INNER_TYPE";
	case T_FUN_SET_TO_STR : return "T_FUN_SET_TO_STR";
	case T_FUN_SET_TO_INNER_TYPE : return "T_FUN_SET_TO_INNER_TYPE";
	case T_FUN_GROUPING : return "T_FUN_GROUPING";
	case T_FUN_SYS_OP_OPNSIZE : return "T_FUN_SYS_OP_OPNSIZE";
	case T_FUN_SYS_PDB_ON : return "T_FUN_SYS_PDB_ON";
	case T_FUN_SYS_PDB_OFF : return "T_FUN_SYS_PDB_OFF";
	case T_FUN_SYS_PDB_INITIALIZE : return "T_FUN_SYS_PDB_INITIALIZE";
	case T_FUN_SYS_PDB_ATTACHSESSION : return "T_FUN_SYS_PDB_ATTACHSESSION";
	case T_FUN_SYS_PDB_DETACHSESSION : return "T_FUN_SYS_PDB_DETACHSESSION";
	case T_FUN_SYS_PDB_SETBREAKPOINT : return "T_FUN_SYS_PDB_SETBREAKPOINT";
	case T_FUN_SYS_PDB_PRINTBACKTRACE : return "T_FUN_SYS_PDB_PRINTBACKTRACE";
	case T_FUN_SYS_PDB_CONTINUE : return "T_FUN_SYS_PDB_CONTINUE";
	case T_FUN_SYS_PDB_SHOWBREAKPOINTS : return "T_FUN_SYS_PDB_SHOWBREAKPOINTS";
	case T_FUN_SYS_PDB_GETVALUE : return "T_FUN_SYS_PDB_GETVALUE";
	case T_FUN_SYS_PDB_DELBREAKPOINT : return "T_FUN_SYS_PDB_DELBREAKPOINT";
	case T_FUN_SYS_PDB_ENABLE_BP : return "T_FUN_SYS_PDB_ENABLE_BP";
	case T_FUN_SYS_PDB_DISABLE_BP : return "T_FUN_SYS_PDB_DISABLE_BP";
	case T_FUN_SYS_PDB_SET_TIMEOUT : return "T_FUN_SYS_PDB_SET_TIMEOUT";
	case T_FUN_SYS_PDB_SET_TIMEOUT_BEHAVIOUR : return "T_FUN_SYS_PDB_SET_TIMEOUT_BEHAVIOUR";
	case T_FUN_SYS_PDB_GET_TIMEOUT_BEHAVIOUR : return "T_FUN_SYS_PDB_GET_TIMEOUT_BEHAVIOUR";
	case T_FUN_SYS_SPM_LOAD_PLANS : return "T_FUN_SYS_SPM_LOAD_PLANS";
	case T_FUN_SYS_SPM_ALTER_BASELINE : return "T_FUN_SYS_SPM_ALTER_BASELINE";
	case T_FUN_SYS_DAY_OF_MONTH : return "T_FUN_SYS_DAY_OF_MONTH";
	case T_FUN_SYS_DAY_OF_WEEK : return "T_FUN_SYS_DAY_OF_WEEK";
	case T_FUN_SYS_DAY_OF_YEAR : return "T_FUN_SYS_DAY_OF_YEAR";
	case T_FUN_SYS_SECOND : return "T_FUN_SYS_SECOND";
	case T_FUN_SYS_MINUTE : return "T_FUN_SYS_MINUTE";
	case T_FUN_SYS_MICROSECOND : return "T_FUN_SYS_MICROSECOND";
	case T_FUN_SYS_TO_SECONDS : return "T_FUN_SYS_TO_SECONDS";
	case T_FUN_SYS_TIME_TO_SEC : return "T_FUN_SYS_TIME_TO_SEC";
	case T_FUN_SYS_SEC_TO_TIME : return "T_FUN_SYS_SEC_TO_TIME";
	case T_FUN_SYS_INTERVAL : return "T_FUN_SYS_INTERVAL";
	case T_FUN_UDF : return "T_FUN_UDF";
	case T_FUN_SYS_SUBTIME : return "T_FUN_SYS_SUBTIME";
	case T_FUN_SYS_SQRT : return "T_FUN_SYS_SQRT";
	case T_FUN_SYS_LOG_TWO : return "T_FUN_SYS_LOG_TWO";
	case T_FUN_SYS_LOG_TEN : return "T_FUN_SYS_LOG_TEN";
	case T_FUN_SYS_SYSDATE : return "T_FUN_SYS_SYSDATE";
	case T_FUN_SYS_TRUNCATE : return "T_FUN_SYS_TRUNCATE";
	case T_FUN_SYS_WEEK_OF_YEAR : return "T_FUN_SYS_WEEK_OF_YEAR";
	case T_FUN_SYS_WEEKDAY_OF_DATE : return "T_FUN_SYS_WEEKDAY_OF_DATE";
	case T_FUN_SYS_YEARWEEK_OF_DATE : return "T_FUN_SYS_YEARWEEK_OF_DATE";
	case T_FUN_SYS_WEEK : return "T_FUN_SYS_WEEK";
	case T_FUN_NORMAL_UDF : return "T_FUN_NORMAL_UDF";
	case T_FUN_AGG_UDF : return "T_FUN_AGG_UDF";
	case T_FUN_SYS_SEQ_NEXTVAL : return "T_FUN_SYS_SEQ_NEXTVAL";
	case T_FUN_SYS_ROWKEY_TO_ROWID : return "T_FUN_SYS_ROWKEY_TO_ROWID";
	case T_FUN_SYS_UID : return "T_FUN_SYS_UID";
	case T_FUN_SYS_LOCALTIMESTAMP : return "T_FUN_SYS_LOCALTIMESTAMP";
	case T_FUN_SYS_SPM_DROP_BASELINE : return "T_FUN_SYS_SPM_DROP_BASELINE";
	case T_FUN_SYS_EXP : return "T_FUN_SYS_EXP";
	case T_FUN_STDDEV_SAMP : return "T_FUN_STDDEV_SAMP";
	case T_FUN_STDDEV_POP : return "T_FUN_STDDEV_POP";
	case T_FUN_STDDEV : return "T_FUN_STDDEV";
	case T_FUN_VARIANCE : return "T_FUN_VARIANCE";
	case T_FUN_SYS_HOUR : return "T_FUN_SYS_HOUR";
	case T_FUN_SYS_STMT_ID : return "T_FUN_SYS_STMT_ID";
	case T_FUN_SYS_REGEXP_INSTR : return "T_FUN_SYS_REGEXP_INSTR";
	case T_FUN_SYS_REGEXP_REPLACE : return "T_FUN_SYS_REGEXP_REPLACE";
	case T_FUN_SYS_REGEXP_LIKE : return "T_FUN_SYS_REGEXP_LIKE";
	case T_FUN_SYS_REGEXP_SUBSTR : return "T_FUN_SYS_REGEXP_SUBSTR";
	case T_FUN_COLUMN_CONV_SKIP_NULL_CHECK : return "T_FUN_COLUMN_CONV_SKIP_NULL_CHECK";
	case T_PDML_PARTITION_ID : return "T_PDML_PARTITION_ID";
	case T_FUN_SYS_MID : return "T_FUN_SYS_MID";
	case T_FUN_SYS_CALC_UROWID : return "T_FUN_SYS_CALC_UROWID";
	case T_FUN_SYS_PDB_GET_RUNTIME_INFO : return "T_FUN_SYS_PDB_GET_RUNTIME_INFO";
	case T_FUN_SYS_REMOVE_CONST : return "T_FUN_SYS_REMOVE_CONST";
	case T_FUN_SYS_PART_KEY_V3 : return "T_FUN_SYS_PART_KEY_V3";
	case T_FUN_SYS_PART_HASH_V2 : return "T_FUN_SYS_PART_HASH_V2";
	case T_FUN_SYS_GREATEST_INNER : return "T_FUN_SYS_GREATEST_INNER";
	case T_FUN_SYS_LEAST_INNER : return "T_FUN_SYS_LEAST_INNER";
	case T_COMMON_FUN_SYS_END : return "T_COMMON_FUN_SYS_END";
	case T_MYSQL_ONLY_SYS_MIN_OP : return "T_MYSQL_ONLY_SYS_MIN_OP";
	case T_FUN_SYS_OB_VERSION : return "T_FUN_SYS_OB_VERSION";
	case T_FUN_SYS_CALC_PARTITION_ID : return "T_FUN_SYS_CALC_PARTITION_ID";
	case T_FUN_SYS_RADIANS : return "T_FUN_SYS_RADIANS";
	case T_FUN_SYS_MAKETIME : return "T_FUN_SYS_MAKETIME";
	case T_FUN_SYS_MONTH_NAME : return "T_FUN_SYS_MONTH_NAME";
	case T_FUN_SYS_FORMAT : return "T_FUN_SYS_FORMAT";
	case T_FUN_SYS_COT : return "T_FUN_SYS_COT";
	case T_FUN_SYS_QUARTER : return "T_FUN_SYS_QUARTER";
	case T_FUN_SYS_BIT_LENGTH : return "T_FUN_SYS_BIT_LENGTH";
	case T_FUN_SYS_PI : return "T_FUN_SYS_PI";
	case T_MYSQL_ONLY_SYS_MAX_OP : return "T_MYSQL_ONLY_SYS_MAX_OP";
	case T_FUN_SYS_CONNECT_BY_PATH : return "T_FUN_SYS_CONNECT_BY_PATH";
	case T_FUN_SYS_SYSTIMESTAMP : return "T_FUN_SYS_SYSTIMESTAMP";
	case T_FUN_SYS_TO_DATE : return "T_FUN_SYS_TO_DATE";
	case T_FUN_SYS_TO_CHAR : return "T_FUN_SYS_TO_CHAR";
	case T_FUN_SYS_TO_NUMBER : return "T_FUN_SYS_TO_NUMBER";
	case T_FUN_SYS_USERENV : return "T_FUN_SYS_USERENV";
	case T_FUN_SYS_SYS_CONTEXT : return "T_FUN_SYS_SYS_CONTEXT";
	case T_FUN_SYS_LENGTHB : return "T_FUN_SYS_LENGTHB";
	case T_FUN_SYS_HEXTORAW : return "T_FUN_SYS_HEXTORAW";
	case T_FUN_SYS_RAWTOHEX : return "T_FUN_SYS_RAWTOHEX";
	case T_FUN_SYS_UTL_RAW_CAST_TO_RAW : return "T_FUN_SYS_UTL_RAW_CAST_TO_RAW";
	case T_FUN_SYS_UTL_RAW_CAST_TO_VARCHAR2 : return "T_FUN_SYS_UTL_RAW_CAST_TO_VARCHAR2";
	case T_FUN_SYS_TO_TIMESTAMP : return "T_FUN_SYS_TO_TIMESTAMP";
	case T_FUN_SYS_TO_TIMESTAMP_TZ : return "T_FUN_SYS_TO_TIMESTAMP_TZ";
	case T_FUN_SYS_UTL_RAW_LENGTH : return "T_FUN_SYS_UTL_RAW_LENGTH";
	case T_FUN_SYS_UTL_RAW_BIT_AND : return "T_FUN_SYS_UTL_RAW_BIT_AND";
	case T_FUN_SYS_UTL_RAW_BIT_OR : return "T_FUN_SYS_UTL_RAW_BIT_OR";
	case T_FUN_SYS_UTL_RAW_BIT_XOR : return "T_FUN_SYS_UTL_RAW_BIT_XOR";
	case T_FUN_SYS_UTL_RAW_BIT_COMPLEMENT : return "T_FUN_SYS_UTL_RAW_BIT_COMPLEMENT";
	case T_FUN_SYS_UTL_RAW_REVERSE : return "T_FUN_SYS_UTL_RAW_REVERSE";
	case T_FUN_SYS_UTL_RAW_COPIES : return "T_FUN_SYS_UTL_RAW_COPIES";
	case T_FUN_SYS_UTL_RAW_COMPARE : return "T_FUN_SYS_UTL_RAW_COMPARE";
	case T_FUN_SYS_UTL_RAW_SUBSTR : return "T_FUN_SYS_UTL_RAW_SUBSTR";
	case T_FUN_SYS_UTL_RAW_CONCAT : return "T_FUN_SYS_UTL_RAW_CONCAT";
	case T_FUN_SYS_TO_CLOB : return "T_FUN_SYS_TO_CLOB";
	case T_FUN_SYS_CHR : return "T_FUN_SYS_CHR";
	case T_FUN_PL_INTEGER_CHECKER : return "T_FUN_PL_INTEGER_CHECKER";
	case T_FUN_PL_GET_CURSOR_ATTR : return "T_FUN_PL_GET_CURSOR_ATTR";
	case T_FUN_PL_ASSOCIATIVE_INDEX : return "T_FUN_PL_ASSOCIATIVE_INDEX";
	case T_FUN_SYS_SESSIONTIMEZONE : return "T_FUN_SYS_SESSIONTIMEZONE";
	case T_FUN_SYS_DBTIMEZONE : return "T_FUN_SYS_DBTIMEZONE";
	case T_FUN_SYS_SYS_EXTRACT_UTC : return "T_FUN_SYS_SYS_EXTRACT_UTC";
	case T_FUN_SYS_TZ_OFFSET : return "T_FUN_SYS_TZ_OFFSET";
	case T_FUN_SYS_FROM_TZ : return "T_FUN_SYS_FROM_TZ";
	case T_FUN_SYS_DBMS_LOB_GETLENGTH : return "T_FUN_SYS_DBMS_LOB_GETLENGTH";
	case T_FUN_SYS_DBMS_LOB_APPEND : return "T_FUN_SYS_DBMS_LOB_APPEND";
	case T_FUN_SYS_DBMS_LOB_READ : return "T_FUN_SYS_DBMS_LOB_READ";
	case T_FUN_SYS_DBMS_LOB_CONVERTTOBLOB : return "T_FUN_SYS_DBMS_LOB_CONVERTTOBLOB";
	case T_FUN_SYS_DBMS_LOB_CAST_CLOB_TO_BLOB : return "T_FUN_SYS_DBMS_LOB_CAST_CLOB_TO_BLOB";
	case T_FUN_SYS_DBMS_LOB_CONVERT_CLOB_CHARSET : return "T_FUN_SYS_DBMS_LOB_CONVERT_CLOB_CHARSET";
	case T_FUN_PL_SQLCODE_SQLERRM : return "T_FUN_PL_SQLCODE_SQLERRM";
	case T_FUN_PL_COLLECTION_CONSTRUCT : return "T_FUN_PL_COLLECTION_CONSTRUCT";
	case T_FUN_SYS_REMAINDER : return "T_FUN_SYS_REMAINDER";
	case T_FUN_PL_OBJECT_CONSTRUCT : return "T_FUN_PL_OBJECT_CONSTRUCT";
	case T_FUN_SYS_ADD_MONTHS : return "T_FUN_SYS_ADD_MONTHS";
	case T_FUN_SYS_LAST_DAY : return "T_FUN_SYS_LAST_DAY";
	case T_FUN_SYS_MONTHS_BETWEEN : return "T_FUN_SYS_MONTHS_BETWEEN";
	case T_FUN_SYS_NEXT_DAY : return "T_FUN_SYS_NEXT_DAY";
	case T_FUN_SYS_TO_DSINTERVAL : return "T_FUN_SYS_TO_DSINTERVAL";
	case T_FUN_SYS_TO_YMINTERVAL : return "T_FUN_SYS_TO_YMINTERVAL";
	case T_FUN_SYS_NUMTODSINTERVAL : return "T_FUN_SYS_NUMTODSINTERVAL";
	case T_FUN_SYS_NUMTOYMINTERVAL : return "T_FUN_SYS_NUMTOYMINTERVAL";
	case T_FUN_SYS_INITCAP : return "T_FUN_SYS_INITCAP";
	case T_FUN_SYS_INSTRB : return "T_FUN_SYS_INSTRB";
	case T_FUN_SYS_SUBSTRB : return "T_FUN_SYS_SUBSTRB";
	case T_FUN_SYS_LENGTHC : return "T_FUN_SYS_LENGTHC";
	case T_FUN_SYS_SUBSTRC : return "T_FUN_SYS_SUBSTRC";
	case T_FUN_SYS_INSTRC : return "T_FUN_SYS_INSTRC";
	case T_FUN_SYS_POWER : return "T_FUN_SYS_POWER";
	case T_FUN_SYS_LN : return "T_FUN_SYS_LN";
	case T_FUN_SYS_LOG : return "T_FUN_SYS_LOG";
	case T_FUN_SYS_WIDTH_BUCKET : return "T_FUN_SYS_WIDTH_BUCKET";
	case T_FUN_SYS_TRANSLATE : return "T_FUN_SYS_TRANSLATE";
	case T_FUN_SYS_ASIN : return "T_FUN_SYS_ASIN";
	case T_FUN_SYS_ACOS : return "T_FUN_SYS_ACOS";
	case T_FUN_SYS_ATAN : return "T_FUN_SYS_ATAN";
	case T_FUN_SYS_ATAN2 : return "T_FUN_SYS_ATAN2";
	case T_FUN_SYS_REGEXP_COUNT : return "T_FUN_SYS_REGEXP_COUNT";
	case T_FUN_NVL2 : return "T_FUN_NVL2";
	case T_FUN_SYS_TO_BINARY_FLOAT : return "T_FUN_SYS_TO_BINARY_FLOAT";
	case T_FUN_SYS_TO_BINARY_DOUBLE : return "T_FUN_SYS_TO_BINARY_DOUBLE";
	case T_FUN_SYS_NANVL : return "T_FUN_SYS_NANVL";
	case T_FUN_SYS_COS : return "T_FUN_SYS_COS";
	case T_FUN_SYS_TAN : return "T_FUN_SYS_TAN";
	case T_FUN_SYS_SIN : return "T_FUN_SYS_SIN";
	case T_FUN_SYS_SINH : return "T_FUN_SYS_SINH";
	case T_FUN_SYS_COSH : return "T_FUN_SYS_COSH";
	case T_FUN_SYS_TANH : return "T_FUN_SYS_TANH";
	case T_FUN_SYS_VSIZE : return "T_FUN_SYS_VSIZE";
	case T_FUN_SYS_ORAHASH : return "T_FUN_SYS_ORAHASH";
	case T_FUN_SYS_PL_SEQ_NEXT_VALUE : return "T_FUN_SYS_PL_SEQ_NEXT_VALUE";
	case T_FUN_SYS_GUID : return "T_FUN_SYS_GUID";
	case T_FUN_SYS_TO_SINGLE_BYTE : return "T_FUN_SYS_TO_SINGLE_BYTE";
	case T_FUN_SYS_TO_MULTI_BYTE : return "T_FUN_SYS_TO_MULTI_BYTE";
	case T_FUN_SYS_UTL_I18N_STRING_TO_RAW : return "T_FUN_SYS_UTL_I18N_STRING_TO_RAW";
	case T_FUN_SYS_UTL_I18N_RAW_TO_CHAR : return "T_FUN_SYS_UTL_I18N_RAW_TO_CHAR";
	case T_FUN_SYS_TO_NCHAR : return "T_FUN_SYS_TO_NCHAR";
	case T_FUN_SYS_SET : return "T_FUN_SYS_SET";
	case T_FUN_SYS_CARDINALITY : return "T_FUN_SYS_CARDINALITY";
	case T_FUN_CORR : return "T_FUN_CORR";
	case T_FUN_COVAR_POP : return "T_FUN_COVAR_POP";
	case T_FUN_COVAR_SAMP : return "T_FUN_COVAR_SAMP";
	case T_FUN_VAR_POP : return "T_FUN_VAR_POP";
	case T_FUN_VAR_SAMP : return "T_FUN_VAR_SAMP";
	case T_FUN_REGR_SLOPE : return "T_FUN_REGR_SLOPE";
	case T_FUN_REGR_INTERCEPT : return "T_FUN_REGR_INTERCEPT";
	case T_FUN_REGR_COUNT : return "T_FUN_REGR_COUNT";
	case T_FUN_REGR_R2 : return "T_FUN_REGR_R2";
	case T_FUN_REGR_AVGX : return "T_FUN_REGR_AVGX";
	case T_FUN_REGR_AVGY : return "T_FUN_REGR_AVGY";
	case T_FUN_REGR_SXX : return "T_FUN_REGR_SXX";
	case T_FUN_REGR_SYY : return "T_FUN_REGR_SYY";
	case T_FUN_REGR_SXY : return "T_FUN_REGR_SXY";
	case T_FUN_SYS_UTL_INADDR_GET_HOST_ADDR : return "T_FUN_SYS_UTL_INADDR_GET_HOST_ADDR";
	case T_FUN_SYS_UTL_INADDR_GET_HOST_NAME : return "T_FUN_SYS_UTL_INADDR_GET_HOST_NAME";
	case T_FUN_GROUP_RANK : return "T_FUN_GROUP_RANK";
	case T_FUN_GROUP_PERCENT_RANK : return "T_FUN_GROUP_PERCENT_RANK";
	case T_FUN_GROUP_DENSE_RANK : return "T_FUN_GROUP_DENSE_RANK";
	case T_FUN_GROUP_CUME_DIST : return "T_FUN_GROUP_CUME_DIST";
	case T_FUN_KEEP_MAX : return "T_FUN_KEEP_MAX";
	case T_FUN_KEEP_MIN : return "T_FUN_KEEP_MIN";
	case T_FUN_KEEP_SUM : return "T_FUN_KEEP_SUM";
	case T_FUN_KEEP_COUNT : return "T_FUN_KEEP_COUNT";
	case T_FUN_KEEP_AVG : return "T_FUN_KEEP_AVG";
	case T_FUN_KEEP_VARIANCE : return "T_FUN_KEEP_VARIANCE";
	case T_FUN_KEEP_STDDEV : return "T_FUN_KEEP_STDDEV";
	case T_FUN_MEDIAN : return "T_FUN_MEDIAN";
	case T_FUN_GROUP_PERCENTILE_CONT : return "T_FUN_GROUP_PERCENTILE_CONT";
	case T_FUN_GROUP_PERCENTILE_DISC : return "T_FUN_GROUP_PERCENTILE_DISC";
	case T_FUN_SYS_USER_CAN_ACCESS_OBJ : return "T_FUN_SYS_USER_CAN_ACCESS_OBJ";
	case T_FUN_EMPTY_CLOB : return "T_FUN_EMPTY_CLOB";
	case T_FUN_EMPTY_BLOB : return "T_FUN_EMPTY_BLOB";
	case T_FUN_SYS_ROWID_TO_CHAR : return "T_FUN_SYS_ROWID_TO_CHAR";
	case T_FUN_SYS_ROWID_TO_NCHAR : return "T_FUN_SYS_ROWID_TO_NCHAR";
	case T_FUN_SYS_CHAR_TO_ROWID : return "T_FUN_SYS_CHAR_TO_ROWID";
	case T_FUN_SYS_TO_BLOB : return "T_FUN_SYS_TO_BLOB";
	case T_FUN_SYS_NLS_LOWER : return "T_FUN_SYS_NLS_LOWER";
	case T_FUN_SYS_NLS_UPPER : return "T_FUN_SYS_NLS_UPPER";
	case T_FUN_KEEP_WM_CONCAT : return "T_FUN_KEEP_WM_CONCAT";
	case T_FUN_WM_CONCAT : return "T_FUN_WM_CONCAT";
	case T_FUN_TOP_FRE_HIST : return "T_FUN_TOP_FRE_HIST";
	case T_FUN_UNISTR : return "T_FUN_UNISTR";
	case T_FUN_PLSQL_VARIABLE : return "T_FUN_PLSQL_VARIABLE";
	case T_FUN_ASCIISTR : return "T_FUN_ASCIISTR";
	case T_FUN_PL_AGG_UDF : return "T_FUN_PL_AGG_UDF";
	case T_FUN_SYS_AT_TIME_ZONE : return "T_FUN_SYS_AT_TIME_ZONE";
	case T_FUN_SYS_AT_LOCAL : return "T_FUN_SYS_AT_LOCAL";
	case T_FUN_HYBRID_HIST : return "T_FUN_HYBRID_HIST";
	case T_FUN_SYS_RAWTONHEX : return "T_FUN_SYS_RAWTONHEX";
	case T_FUN_SYS_TIMESTAMP : return "T_FUN_SYS_TIMESTAMP";
	case T_FUN_SYS_GET_FORMAT : return "T_FUN_SYS_GET_FORMAT";
	case T_FUN_SYS_MAKEDATE : return "T_FUN_SYS_MAKEDATE";
	case T_FUN_SYS_PERIOD_ADD : return "T_FUN_SYS_PERIOD_ADD";
	case T_FUN_SYS_UTC_TIME : return "T_FUN_SYS_UTC_TIME";
	case T_FUN_SYS_UTC_DATE : return "T_FUN_SYS_UTC_DATE";
	case T_FUN_SYS_TIME_FORMAT : return "T_FUN_SYS_TIME_FORMAT";
	case T_FUN_SYS_END : return "T_FUN_SYS_END";
	case T_MAX_OP : return "T_MAX_OP";
	case T_FUN_MATCH_AGAINST : return "T_FUN_MATCH_AGAINST";
	case T_ROW_ITER_EXPR : return "T_ROW_ITER_EXPR";
	case T_DEFAULT : return "T_DEFAULT";
	case T_CAST_ARGUMENT : return "T_CAST_ARGUMENT";
	case T_CHAR_CHARSET : return "T_CHAR_CHARSET";
	case T_HINT : return "T_HINT";  // Hint message from rowkey
	case T_IDENT : return "T_IDENT";
	case T_SIMPLE_IDENT : return "T_SIMPLE_IDENT";
	case T_STAR : return "T_STAR";
	case T_SYSTEM_VARIABLE : return "T_SYSTEM_VARIABLE";
	case T_USER_VARIABLE_IDENTIFIER : return "T_USER_VARIABLE_IDENTIFIER";
	case T_LEFT_VALUE : return "T_LEFT_VALUE";  // assignment
	case T_TYPE_INTEGER : return "T_TYPE_INTEGER";
	case T_TYPE_FLOAT : return "T_TYPE_FLOAT";
	case T_TYPE_DOUBLE : return "T_TYPE_DOUBLE";
	case T_TYPE_DECIMAL : return "T_TYPE_DECIMAL";
	case T_TYPE_BOOLEAN : return "T_TYPE_BOOLEAN";
	case T_TYPE_YEAR : return "T_TYPE_YEAR";
	case T_TYPE_DATE : return "T_TYPE_DATE";
	case T_TYPE_TIME : return "T_TYPE_TIME";
	case T_TYPE_DATETIME : return "T_TYPE_DATETIME";
	case T_TYPE_TIMESTAMP : return "T_TYPE_TIMESTAMP";
	case T_TYPE_CHARACTER : return "T_TYPE_CHARACTER";
	case T_TYPE_VARCHAR : return "T_TYPE_VARCHAR";
	case T_TYPE_NUMBER : return "T_TYPE_NUMBER";
	case T_COLUMN_REF : return "T_COLUMN_REF";
	case T_RELATION_FACTOR : return "T_RELATION_FACTOR";
	case T_RELATION_FACTOR_IN_HINT : return "T_RELATION_FACTOR_IN_HINT";
	case T_RELATION_FACTOR_IN_HINT_LIST : return "T_RELATION_FACTOR_IN_HINT_LIST";
	case T_INSERT : return "T_INSERT";
	case T_INSERT_INTO_CLAUSE : return "T_INSERT_INTO_CLAUSE";
	case T_VALUE_LIST : return "T_VALUE_LIST";
	case T_SINGLE_TABLE_INSERT : return "T_SINGLE_TABLE_INSERT";
	case T_DELETE : return "T_DELETE";
	case T_SELECT : return "T_SELECT";
	case T_UPDATE : return "T_UPDATE";
	case T_MERGE : return "T_MERGE";
	case T_REPLACE : return "T_REPLACE";
	case T_EXPLAIN : return "T_EXPLAIN";
	case T_LINK_NODE : return "T_LINK_NODE";
	case T_ASSIGN_LIST : return "T_ASSIGN_LIST";
	case T_ASSIGN_ITEM : return "T_ASSIGN_ITEM";
	case T_STMT_LIST : return "T_STMT_LIST";
	case T_EXPR_LIST : return "T_EXPR_LIST";
	case T_WHEN_LIST : return "T_WHEN_LIST";
	case T_PROJECT_LIST : return "T_PROJECT_LIST";
	case T_PROJECT_ITEM : return "T_PROJECT_ITEM";
	case T_FROM_LIST : return "T_FROM_LIST";
	case T_TABLE_REFERENCES : return "T_TABLE_REFERENCES";
	case T_DELETE_TABLE_NODE : return "T_DELETE_TABLE_NODE";
	case T_SET_UNION : return "T_SET_UNION";
	case T_SET_INTERSECT : return "T_SET_INTERSECT";
	case T_SET_EXCEPT : return "T_SET_EXCEPT";
	case T_CONNECT_BY_CLAUSE : return "T_CONNECT_BY_CLAUSE";
	case T_NOCYCLE : return "T_NOCYCLE";
	case T_SIBLINGS : return "T_SIBLINGS";
	case T_WHERE_CLAUSE : return "T_WHERE_CLAUSE";
	case T_GROUPBY_CLAUSE : return "T_GROUPBY_CLAUSE";
	case T_GROUPBY_LIST : return "T_GROUPBY_LIST";
	case T_GROUPBY_KEY : return "T_GROUPBY_KEY";
	case T_ROLLUP : return "T_ROLLUP";
	case T_LIMIT_CLAUSE : return "T_LIMIT_CLAUSE";
	case T_COMMA_LIMIT_CLAUSE : return "T_COMMA_LIMIT_CLAUSE";
	case T_LIKE_CLAUSE : return "T_LIKE_CLAUSE";
	case T_SEPARATOR_CLAUSE : return "T_SEPARATOR_CLAUSE";
	case T_INTO_CLAUSE : return "T_INTO_CLAUSE";
	case T_INTO_OUTFILE : return "T_INTO_OUTFILE";
	case T_INTO_DUMPFILE : return "T_INTO_DUMPFILE";
	case T_INTO_VARIABLES : return "T_INTO_VARIABLES";
	case T_SORT_LIST : return "T_SORT_LIST";
	case T_SORT_KEY : return "T_SORT_KEY";
	case T_SORT_ASC : return "T_SORT_ASC";
	case T_SORT_DESC : return "T_SORT_DESC";
	case T_ROLLUP_KEY : return "T_ROLLUP_KEY";
	case T_ROLLUP_DELIMIT : return "T_ROLLUP_DELIMIT";
	case T_ALL : return "T_ALL";
	case T_ANY : return "T_ANY";
	case T_DISTINCT : return "T_DISTINCT";
	case T_ORG : return "T_ORG";
	case T_ALIAS : return "T_ALIAS";
	case T_PROJECT_STRING : return "T_PROJECT_STRING";
	case T_COLUMN_LIST : return "T_COLUMN_LIST";
	case T_VALUE_VECTOR : return "T_VALUE_VECTOR";
	case T_JOINED_TABLE : return "T_JOINED_TABLE";
	case T_JOIN_INNER : return "T_JOIN_INNER";
	case T_JOIN_FULL : return "T_JOIN_FULL";
	case T_JOIN_LEFT : return "T_JOIN_LEFT";
	case T_JOIN_RIGHT : return "T_JOIN_RIGHT";
	case T_JOIN_CROSS : return "T_JOIN_CROSS";
	case T_NATURAL_JOIN : return "T_NATURAL_JOIN";
	case T_STRAIGHT_JOIN : return "T_STRAIGHT_JOIN";
	case T_CASE : return "T_CASE";
	case T_WHEN : return "T_WHEN";
	case T_CREATE_TABLE : return "T_CREATE_TABLE";
	case T_CREATE_TABLE_LIKE : return "T_CREATE_TABLE_LIKE";
	case T_CREATE_VIEW : return "T_CREATE_VIEW";
	case T_ALTER_VIEW : return "T_ALTER_VIEW";
	case T_MATERIALIZED : return "T_MATERIALIZED";
	case T_TABLE_ELEMENT_LIST : return "T_TABLE_ELEMENT_LIST";
	case T_TABLE_OPTION_LIST : return "T_TABLE_OPTION_LIST";
	case T_PRIMARY_KEY : return "T_PRIMARY_KEY";
	case T_SPLIT_KEY : return "T_SPLIT_KEY";
	case T_COLUMN_DEFINITION : return "T_COLUMN_DEFINITION";
	case T_COLUMN_ATTRIBUTES : return "T_COLUMN_ATTRIBUTES";
	case T_CONSTR_NOT_NULL : return "T_CONSTR_NOT_NULL";
	case T_CONSTR_NULL : return "T_CONSTR_NULL";
	case T_CONSTR_DEFAULT : return "T_CONSTR_DEFAULT";
	case T_CONSTR_ORIG_DEFAULT : return "T_CONSTR_ORIG_DEFAULT";
	case T_CONSTR_AUTO_INCREMENT : return "T_CONSTR_AUTO_INCREMENT";
	case T_CONSTR_PRIMARY_KEY : return "T_CONSTR_PRIMARY_KEY";
	case T_CONSTR_UNIQUE_KEY : return "T_CONSTR_UNIQUE_KEY";
	case T_CHECK_CONSTRAINT : return "T_CHECK_CONSTRAINT";
	case T_WINDOW_FUNCTION : return "T_WINDOW_FUNCTION";
	case T_WIN_GENERALIZED_WINDOW : return "T_WIN_GENERALIZED_WINDOW";
	case T_WIN_NEW_GENERALIZED_WINDOW : return "T_WIN_NEW_GENERALIZED_WINDOW";
	case T_WIN_WINDOW : return "T_WIN_WINDOW";
	case T_WIN_NAMED_WINDOWS : return "T_WIN_NAMED_WINDOWS";
	case T_WIN_NAMED_WINDOW : return "T_WIN_NAMED_WINDOW";
	case T_WIN_BOUND : return "T_WIN_BOUND";
	case T_WIN_INTERVAL : return "T_WIN_INTERVAL";
	case T_WIN_FUN_ROW_NUMBER : return "T_WIN_FUN_ROW_NUMBER";
	case T_WIN_FUN_RANK : return "T_WIN_FUN_RANK";
	case T_WIN_FUN_DENSE_RANK : return "T_WIN_FUN_DENSE_RANK";
	case T_WIN_FUN_PERCENT_RANK : return "T_WIN_FUN_PERCENT_RANK";
	case T_WIN_FUN_SUM : return "T_WIN_FUN_SUM";
	case T_WIN_FUN_MAX : return "T_WIN_FUN_MAX";
	case T_WIN_FUN_NTILE : return "T_WIN_FUN_NTILE";
	case T_WIN_FUN_NTH_VALUE : return "T_WIN_FUN_NTH_VALUE";
	case T_WIN_FUN_FIRST_VALUE : return "T_WIN_FUN_FIRST_VALUE";
	case T_WIN_FUN_LAST_VALUE : return "T_WIN_FUN_LAST_VALUE";
	case T_WIN_FUN_CUME_DIST : return "T_WIN_FUN_CUME_DIST";
	case T_WIN_FUN_LEAD : return "T_WIN_FUN_LEAD";
	case T_WIN_FUN_LAG : return "T_WIN_FUN_LAG";
	case T_WIN_FUN_AVG : return "T_WIN_FUN_AVG";
	case T_WIN_FUN_RATIO_TO_REPORT : return "T_WIN_FUN_RATIO_TO_REPORT";
	case T_RESPECT : return "T_RESPECT";
	case T_FIRST : return "T_FIRST";
	case T_LAST : return "T_LAST";
	case T_VIRTUAL_COLUMN : return "T_VIRTUAL_COLUMN";
	case T_STORED_COLUMN : return "T_STORED_COLUMN";
	case T_IF_NOT_EXISTS : return "T_IF_NOT_EXISTS";
	case T_IF_EXISTS : return "T_IF_EXISTS";
	case T_JOIN_INFO : return "T_JOIN_INFO";
	case T_EXPIRE_INFO : return "T_EXPIRE_INFO";
	case T_TABLET_MAX_SIZE : return "T_TABLET_MAX_SIZE";
	case T_BLOCK_SIZE : return "T_BLOCK_SIZE";
	case T_TABLE_ID : return "T_TABLE_ID";
	case T_DATA_TABLE_ID : return "T_DATA_TABLE_ID";
	case T_INDEX_TABLE_ID : return "T_INDEX_TABLE_ID";
	case T_VIRTUAL_COLUMN_ID : return "T_VIRTUAL_COLUMN_ID";
	case T_DATABASE_ID : return "T_DATABASE_ID";
	case T_REPLICA_NUM : return "T_REPLICA_NUM";
	case T_RESTORE_TENANT : return "T_RESTORE_TENANT";
	case T_TABLET_SIZE : return "T_TABLET_SIZE";
	case T_LOCALITY : return "T_LOCALITY";
	case T_DUPLICATE_SCOPE : return "T_DUPLICATE_SCOPE";
	case T_LOGONLY_REPLICA_NUM : return "T_LOGONLY_REPLICA_NUM";
	case T_TABLEGROUP : return "T_TABLEGROUP";
	case T_COMPRESSION : return "T_COMPRESSION";
	case T_PROGRESSIVE_MERGE_NUM : return "T_PROGRESSIVE_MERGE_NUM";
	case T_COMMENT : return "T_COMMENT";
	case T_COLUMN_ID : return "T_COLUMN_ID";
	case T_ON_UPDATE : return "T_ON_UPDATE";
	case T_USE_BLOOM_FILTER : return "T_USE_BLOOM_FILTER";
	case T_CHARSET : return "T_CHARSET";
	case T_COLLATION : return "T_COLLATION";
	case T_INDEX_SCOPE : return "T_INDEX_SCOPE";
	case T_DROP_TABLE : return "T_DROP_TABLE";
	case T_TABLE_LIST : return "T_TABLE_LIST";
	case T_DROP_VIEW : return "T_DROP_VIEW";
	case T_VIEW_LIST : return "T_VIEW_LIST";
	case T_PARTITION_OPTION : return "T_PARTITION_OPTION";
	case T_HASH_PARTITION : return "T_HASH_PARTITION";
	case T_KEY_PARTITION : return "T_KEY_PARTITION";
	case T_RANGE_PARTITION : return "T_RANGE_PARTITION";
	case T_RANGE_COLUMNS_PARTITION : return "T_RANGE_COLUMNS_PARTITION";
	case T_LIST_PARTITION : return "T_LIST_PARTITION";
	case T_LIST_COLUMNS_PARTITION : return "T_LIST_COLUMNS_PARTITION";
	case T_VERTICAL_COLUMNS_PARTITION : return "T_VERTICAL_COLUMNS_PARTITION";
	case T_AUX_COLUMN_LIST : return "T_AUX_COLUMN_LIST";
	case T_ADDR_PARTITION : return "T_ADDR_PARTITION";
	case T_PARTITION_LIST : return "T_PARTITION_LIST";
	case T_PARTITION_ELEMENT : return "T_PARTITION_ELEMENT";
	case T_PART_ID : return "T_PART_ID";
	case T_CREATE_SEQUENCE : return "T_CREATE_SEQUENCE";
	case T_DROP_SEQUENCE : return "T_DROP_SEQUENCE";
	case T_ALTER_SEQUENCE : return "T_ALTER_SEQUENCE";
	case T_SEQUENCE_OPTION_LIST : return "T_SEQUENCE_OPTION_LIST";
	case T_INCREMENT_BY : return "T_INCREMENT_BY";
	case T_START_WITH : return "T_START_WITH";
	case T_MAXVALUE : return "T_MAXVALUE";
	case T_NOMAXVALUE : return "T_NOMAXVALUE";
	case T_MINVALUE : return "T_MINVALUE";
	case T_NOMINVALUE : return "T_NOMINVALUE";
	case T_CYCLE : return "T_CYCLE";
  /*	case T_NOCYCLE : return "T_NOCYCLE"; defined */
	case T_ORDER : return "T_ORDER";
	case T_NOORDER : return "T_NOORDER";
	case T_CACHE : return "T_CACHE";
	case T_NOCACHE : return "T_NOCACHE";
	case T_SIGNED : return "T_SIGNED";
	case T_UNSIGNED : return "T_UNSIGNED";
	case T_ZEROFILL : return "T_ZEROFILL";
	case T_USE_PARTITION : return "T_USE_PARTITION";
	case T_AUTO_INCREMENT : return "T_AUTO_INCREMENT";
	case T_USING_HASH : return "T_USING_HASH";
	case T_USING_BTREE : return "T_USING_BTREE";
	case T_WITH_CLAUSE_AS : return "T_WITH_CLAUSE_AS";
	case T_WITH_CLAUSE_LIST : return "T_WITH_CLAUSE_LIST";
	case T_CYCLE_NODE : return "T_CYCLE_NODE";
	case T_SEARCH_BREADTH_NODE : return "T_SEARCH_BREADTH_NODE";
	case T_SEARCH_DEPTH_NODE : return "T_SEARCH_DEPTH_NODE";
	case T_EXPR_WITH_ALIAS : return "T_EXPR_WITH_ALIAS";
	case T_TABLE_COLLECTION_EXPRESSION : return "T_TABLE_COLLECTION_EXPRESSION";
	case T_PSEUDO_COLUMN : return "T_PSEUDO_COLUMN";
	case T_LEVEL : return "T_LEVEL";
	case T_CONNECT_BY_ISCYCLE : return "T_CONNECT_BY_ISCYCLE";
	case T_CONNECT_BY_ISLEAF : return "T_CONNECT_BY_ISLEAF";
	case T_CTE_SEARCH_COLUMN : return "T_CTE_SEARCH_COLUMN";
	case T_CTE_CYCLE_COLUMN : return "T_CTE_CYCLE_COLUMN";
	case T_RENAME_TABLE : return "T_RENAME_TABLE";
	case T_RENAME_TABLE_ACTION : return "T_RENAME_TABLE_ACTION";
	case T_TRUNCATE_TABLE : return "T_TRUNCATE_TABLE";
	case T_ALTER_TABLE : return "T_ALTER_TABLE";
	case T_ALTER_TABLE_OPTION : return "T_ALTER_TABLE_OPTION";
	case T_ALTER_INDEX_OPTION : return "T_ALTER_INDEX_OPTION";
	case T_ALTER_CHECK_CONSTRAINT_OPTION : return "T_ALTER_CHECK_CONSTRAINT_OPTION";
	case T_ALTER_PARTITION_OPTION : return "T_ALTER_PARTITION_OPTION";
	case T_ALTER_PARTITION_DROP : return "T_ALTER_PARTITION_DROP";
	case T_ALTER_PARTITION_ADD : return "T_ALTER_PARTITION_ADD";
	case T_ALTER_COLUMN_OPTION : return "T_ALTER_COLUMN_OPTION";
	case T_ALTER_TABLEGROUP_OPTION : return "T_ALTER_TABLEGROUP_OPTION";
	case T_ALTER_TABLE_ACTION_LIST : return "T_ALTER_TABLE_ACTION_LIST";
	case T_COLUMN_ADD : return "T_COLUMN_ADD";
	case T_COLUMN_ADD_BEFORE : return "T_COLUMN_ADD_BEFORE";
	case T_COLUMN_ADD_AFTER : return "T_COLUMN_ADD_AFTER";
	case T_COLUMN_ADD_FIRST : return "T_COLUMN_ADD_FIRST";
	case T_TABLE_RENAME : return "T_TABLE_RENAME";
	case T_COLUMN_DROP : return "T_COLUMN_DROP";
	case T_COLUMN_ALTER : return "T_COLUMN_ALTER";
	case T_COLUMN_RENAME : return "T_COLUMN_RENAME";
	case T_COLUMN_MODIFY : return "T_COLUMN_MODIFY";
	case T_COLUMN_CHANGE : return "T_COLUMN_CHANGE";
	case T_INDEX_ADD : return "T_INDEX_ADD";
	case T_INDEX_DROP : return "T_INDEX_DROP";
	case T_INDEX_ALTER : return "T_INDEX_ALTER";
	case T_INDEX_RENAME : return "T_INDEX_RENAME";
	case T_VISIBLE : return "T_VISIBLE";
	case T_INVISIBLE : return "T_INVISIBLE";
	case T_TABLEGROUP_DROP : return "T_TABLEGROUP_DROP";
	case T_ORDER_BY : return "T_ORDER_BY";
	case T_SET_TABLE_COMMENT : return "T_SET_TABLE_COMMENT";
	case T_SET_COLUMN_COMMENT : return "T_SET_COLUMN_COMMENT";
	case T_CREATE_TABLEGROUP : return "T_CREATE_TABLEGROUP";
	case T_DROP_TABLEGROUP : return "T_DROP_TABLEGROUP";
	case T_ALTER_TABLEGROUP : return "T_ALTER_TABLEGROUP";
	case T_DEFAULT_TABLEGROUP : return "T_DEFAULT_TABLEGROUP";
	case T_TEMPORARY : return "T_TEMPORARY";
	case T_ALTER_SYSTEM : return "T_ALTER_SYSTEM";
	case T_CHANGE_OBI : return "T_CHANGE_OBI";
	case T_FORCE : return "T_FORCE";
	case T_USE : return "T_USE";
	case T_IGNORE : return "T_IGNORE";
	case T_INDEX_HINT_LIST : return "T_INDEX_HINT_LIST";
	case T_INDEX_HINT_DEF : return "T_INDEX_HINT_DEF";
	case T_SET_MASTER : return "T_SET_MASTER";
	case T_SET_SLAVE : return "T_SET_SLAVE";
	case T_SET_MASTER_SLAVE : return "T_SET_MASTER_SLAVE";
	case T_SYTEM_ACTION_LIST : return "T_SYTEM_ACTION_LIST";
	case T_SYSTEM_ACTION : return "T_SYSTEM_ACTION";
	case T_CLUSTER : return "T_CLUSTER";
	case T_SERVER_INFO : return "T_SERVER_INFO";
	case T_SERVER_INFO_LIST : return "T_SERVER_INFO_LIST";
	case T_SERVER_LIST : return "T_SERVER_LIST";
	case T_CLUSTER_V2 : return "T_CLUSTER_V2";
	case T_SERVER_ADDRESS : return "T_SERVER_ADDRESS";
	case T_SWITCH_MASTER_LIST : return "T_SWITCH_MASTER_LIST";
	case T_SWITCH_MASTER : return "T_SWITCH_MASTER";
	case T_SIWTCH_ACTION : return "T_SIWTCH_ACTION";
	case T_SERVER_ACTION : return "T_SERVER_ACTION";
	case T_ZONE_ACTION : return "T_ZONE_ACTION";
	case T_IP_PORT : return "T_IP_PORT";
	case T_SHOW_TABLES : return "T_SHOW_TABLES";
	case T_SHOW_DATABASES : return "T_SHOW_DATABASES";
	case T_SHOW_VARIABLES : return "T_SHOW_VARIABLES";
	case T_SHOW_COLUMNS : return "T_SHOW_COLUMNS";
	case T_SHOW_SCHEMA : return "T_SHOW_SCHEMA";
	case T_SHOW_CREATE_DATABASE : return "T_SHOW_CREATE_DATABASE";
	case T_SHOW_CREATE_TABLE : return "T_SHOW_CREATE_TABLE";
	case T_SHOW_CREATE_VIEW : return "T_SHOW_CREATE_VIEW";
	case T_SHOW_TABLE_STATUS : return "T_SHOW_TABLE_STATUS";
	case T_SHOW_PARAMETERS : return "T_SHOW_PARAMETERS";
	case T_SHOW_INDEXES : return "T_SHOW_INDEXES";
	case T_SHOW_PROCESSLIST : return "T_SHOW_PROCESSLIST";
	case T_SHOW_SERVER_STATUS : return "T_SHOW_SERVER_STATUS";
	case T_SHOW_WARNINGS : return "T_SHOW_WARNINGS";
	case T_SHOW_ERRORS : return "T_SHOW_ERRORS";
	case T_SHOW_CHARSET : return "T_SHOW_CHARSET";
	case T_SHOW_COLLATION : return "T_SHOW_COLLATION";
	case T_SHOW_TABLEGROUPS : return "T_SHOW_TABLEGROUPS";
	case T_SHOW_STATUS : return "T_SHOW_STATUS";
	case T_SHOW_TENANT : return "T_SHOW_TENANT";
	case T_SHOW_CREATE_TENANT : return "T_SHOW_CREATE_TENANT";
	case T_SHOW_TRACE : return "T_SHOW_TRACE";
	case T_SHOW_ENGINES : return "T_SHOW_ENGINES";
	case T_SHOW_PRIVILEGES : return "T_SHOW_PRIVILEGES";
	case T_SHOW_RECYCLEBIN : return "T_SHOW_RECYCLEBIN";
	case T_SHOW_CREATE_PROCEDURE : return "T_SHOW_CREATE_PROCEDURE";
	case T_SHOW_CREATE_FUNCTION : return "T_SHOW_CREATE_FUNCTION";
	case T_SHOW_PROCEDURE_STATUS : return "T_SHOW_PROCEDURE_STATUS";
	case T_SHOW_FUNCTION_STATUS : return "T_SHOW_FUNCTION_STATUS";
	case T_SHOW_CREATE_TABLEGROUP : return "T_SHOW_CREATE_TABLEGROUP";
	case T_SHOW_GRANTS : return "T_SHOW_GRANTS";
	case T_SHOW_LIMIT : return "T_SHOW_LIMIT";
	case T_CREATE_USER : return "T_CREATE_USER";
	case T_CREATE_USER_SPEC : return "T_CREATE_USER_SPEC";
	case T_DROP_USER : return "T_DROP_USER";
	case T_SET_PASSWORD : return "T_SET_PASSWORD";
	case T_RENAME_USER : return "T_RENAME_USER";
	case T_RENAME_INFO : return "T_RENAME_INFO";
	case T_LOCK_USER : return "T_LOCK_USER";
	case T_GRANT : return "T_GRANT";
	case T_PRIVILEGES : return "T_PRIVILEGES";
	case T_PRIV_LEVEL : return "T_PRIV_LEVEL";
	case T_PRIV_TYPE : return "T_PRIV_TYPE";
	case T_USERS : return "T_USERS";
	case T_REVOKE : return "T_REVOKE";
	case T_REVOKE_ALL : return "T_REVOKE_ALL";
	case T_BEGIN : return "T_BEGIN";
	case T_COMMIT : return "T_COMMIT";
	case T_PREPARE : return "T_PREPARE";
	case T_DEALLOCATE : return "T_DEALLOCATE";
	case T_EXECUTE : return "T_EXECUTE";
	case T_ARGUMENT_LIST : return "T_ARGUMENT_LIST";
	case T_VARIABLE_SET : return "T_VARIABLE_SET";
	case T_VAR_VAL : return "T_VAR_VAL";
	case T_ROLLBACK : return "T_ROLLBACK";
	case T_ANY_HOST_NAME : return "T_ANY_HOST_NAME";
	case T_USER_WITH_HOST_NAME : return "T_USER_WITH_HOST_NAME";
	case T_INDEX_COLUMN_LIST : return "T_INDEX_COLUMN_LIST";
	case T_INDEX_USING_ALGORITHM : return "T_INDEX_USING_ALGORITHM";
	case T_CREATE_INDEX : return "T_CREATE_INDEX";
	case T_SORT_COLUMN_KEY : return "T_SORT_COLUMN_KEY";
	case T_STORING : return "T_STORING";
	case T_STORING_COLUMN_LIST : return "T_STORING_COLUMN_LIST";
	case T_WITH_ROWID : return "T_WITH_ROWID";
	case T_FULLTEXT_COLUMN_LIST : return "T_FULLTEXT_COLUMN_LIST";
	case T_MATCH_COLUMN_LIST : return "T_MATCH_COLUMN_LIST";
	case T_PARSER_NAME : return "T_PARSER_NAME";
	case T_DROP_INDEX : return "T_DROP_INDEX";
	case T_FLASHBACK_TENANT : return "T_FLASHBACK_TENANT";
	case T_FLASHBACK_DATABASE : return "T_FLASHBACK_DATABASE";
	case T_FLASHBACK_TABLE_FROM_RECYCLEBIN : return "T_FLASHBACK_TABLE_FROM_RECYCLEBIN";
	case T_FLASHBACK_INDEX : return "T_FLASHBACK_INDEX";
	case T_PURGE_RECYCLEBIN : return "T_PURGE_RECYCLEBIN";
	case T_PURGE_TENANT : return "T_PURGE_TENANT";
	case T_PURGE_DATABASE : return "T_PURGE_DATABASE";
	case T_PURGE_TABLE : return "T_PURGE_TABLE";
	case T_PURGE_INDEX : return "T_PURGE_INDEX";
	case T_HINT_OPTION_LIST : return "T_HINT_OPTION_LIST";
	case T_READ_CONSISTENCY : return "T_READ_CONSISTENCY";
	case T_HOTSPOT : return "T_HOTSPOT";
	case T_INDEX : return "T_INDEX";
	case T_INDEX_NAME : return "T_INDEX_NAME";
	case T_FROZEN_VERSION : return "T_FROZEN_VERSION";
	case T_TOPK : return "T_TOPK";
	case T_CLIENT_VERSION : return "T_CLIENT_VERSION";
	case T_MYSQL_DRIVER : return "T_MYSQL_DRIVER";
	case T_QUERY_TIMEOUT : return "T_QUERY_TIMEOUT";
	case T_LOG_LEVEL : return "T_LOG_LEVEL";
	case T_LEADING : return "T_LEADING";
	case T_ORDERED : return "T_ORDERED";
	case T_FULL : return "T_FULL";
	case T_USE_MERGE : return "T_USE_MERGE";
	case T_USE_HASH : return "T_USE_HASH";
	case T_NO_USE_HASH : return "T_NO_USE_HASH";
	case T_USE_NL : return "T_USE_NL";
	case T_USE_PLAN_CACHE : return "T_USE_PLAN_CACHE";
	case T_USE_JIT : return "T_USE_JIT";
	case T_NO_USE_JIT : return "T_NO_USE_JIT";
	case T_USE_HASH_AGGREGATE : return "T_USE_HASH_AGGREGATE";
	case T_NO_USE_HASH_AGGREGATE : return "T_NO_USE_HASH_AGGREGATE";
	case T_NEG_SIGN : return "T_NEG_SIGN";
	case T_USE_LATE_MATERIALIZATION : return "T_USE_LATE_MATERIALIZATION";
	case T_NO_USE_LATE_MATERIALIZATION : return "T_NO_USE_LATE_MATERIALIZATION";
	case T_NO_REWRITE : return "T_NO_REWRITE";
	case T_TRACE_LOG : return "T_TRACE_LOG";
	case T_USE_BNL : return "T_USE_BNL";
	case T_QB_NAME : return "T_QB_NAME";
	case T_MAX_CONCURRENT : return "T_MAX_CONCURRENT";
	case T_PARALLEL : return "T_PARALLEL";
	case T_NO_PARALLEL : return "T_NO_PARALLEL";
	case T_PQ_DISTRIBUTE : return "T_PQ_DISTRIBUTE";
	case T_DISTRIBUTE_BROADCAST : return "T_DISTRIBUTE_BROADCAST";
	case T_DISTRIBUTE_HASH : return "T_DISTRIBUTE_HASH";
	case T_DISTRIBUTE_NONE : return "T_DISTRIBUTE_NONE";
	case T_DISTRIBUTE_PARTITION : return "T_DISTRIBUTE_PARTITION";
	case T_DISTRIBUTE_RANDOM_LOCAL : return "T_DISTRIBUTE_RANDOM_LOCAL";
	case T_USE_PX : return "T_USE_PX";
	case T_PX_JOIN_FILTER : return "T_PX_JOIN_FILTER";
	case T_TRACING : return "T_TRACING";
	case T_STAT : return "T_STAT";
	case T_KILL : return "T_KILL";
	case T_CANCEL : return "T_CANCEL";
	case T_CANCEL_TASK : return "T_CANCEL_TASK";
	case T_BOOTSTRAP : return "T_BOOTSTRAP";
	case T_FLUSH_CACHE : return "T_FLUSH_CACHE";
	case T_FLUSH_KVCACHE : return "T_FLUSH_KVCACHE";
	case T_FLUSH_ILOGCACHE : return "T_FLUSH_ILOGCACHE";
	case T_ADMIN_SERVER : return "T_ADMIN_SERVER";
	case T_ADMIN_ZONE : return "T_ADMIN_ZONE";
	case T_CS_DISKMAINTAIN : return "T_CS_DISKMAINTAIN";
	case T_DISK_OP_PARAM : return "T_DISK_OP_PARAM";
	case T_CS_ADMIN_PARAMS : return "T_CS_ADMIN_PARAMS";
	case T_CREATE_TABLET : return "T_CREATE_TABLET";
	case T_DROP_REPLICA : return "T_DROP_REPLICA";
	case T_MIGRATE_REPLICA : return "T_MIGRATE_REPLICA";
	case T_CHECK_TABLET : return "T_CHECK_TABLET";
	case T_REPORT_REPLICA : return "T_REPORT_REPLICA";
	case T_IP_PORT_CLUSTER : return "T_IP_PORT_CLUSTER";
	case T_RANGE : return "T_RANGE";
	case T_VERSION : return "T_VERSION";
	case T_IP_SOURCE : return "T_IP_SOURCE";
	case T_IP_DESTINATION : return "T_IP_DESTINATION";
	case T_SWITCH_ROOTSERVER : return "T_SWITCH_ROOTSERVER";
	case T_SWITCH_UPDATESERVER : return "T_SWITCH_UPDATESERVER";
	case T_CLUSTER_MANAGER : return "T_CLUSTER_MANAGER";
	case T_CLUSTER_ID : return "T_CLUSTER_ID";
	case T_FREEZE : return "T_FREEZE";
	case T_DROP_MEMTABLE : return "T_DROP_MEMTABLE";
	case T_CLEAR_MEMTABLE : return "T_CLEAR_MEMTABLE";
	case T_DELAY : return "T_DELAY";
	case T_PRINT_ROOT_TABLE : return "T_PRINT_ROOT_TABLE";
	case T_ADD_UPDATESERVER : return "T_ADD_UPDATESERVER";
	case T_DELETE_UPDATESERVER : return "T_DELETE_UPDATESERVER";
	case T_CHECK_ROOT_TABLE : return "T_CHECK_ROOT_TABLE";
	case T_CLEAR_ROOT_TABLE : return "T_CLEAR_ROOT_TABLE";
	case T_REFRESH_SCHEMA : return "T_REFRESH_SCHEMA";
	case T_REFRESH_MEMORY_STAT : return "T_REFRESH_MEMORY_STAT";
	case T_NAME_LIST : return "T_NAME_LIST";
	case T_CREATE_DATABASE : return "T_CREATE_DATABASE";
	case T_DROP_DATABASE : return "T_DROP_DATABASE";
	case T_ALTER_DATABASE : return "T_ALTER_DATABASE";
	case T_USE_DATABASE : return "T_USE_DATABASE";
	case T_DATABASE_OPTION_LIST : return "T_DATABASE_OPTION_LIST";
	case T_CREATE_TENANT : return "T_CREATE_TENANT";
	case T_DROP_TENANT : return "T_DROP_TENANT";
	case T_MODIFY_TENANT : return "T_MODIFY_TENANT";
	case T_LOCK_TENANT : return "T_LOCK_TENANT";
	case T_ZONE_LIST : return "T_ZONE_LIST";
	case T_RESOURCE_POOL_LIST : return "T_RESOURCE_POOL_LIST";
	case T_ZONE : return "T_ZONE";
	case T_TENANT_RESOURCE_POOL_LIST : return "T_TENANT_RESOURCE_POOL_LIST";
	case T_TENANT_OPTION_LIST : return "T_TENANT_OPTION_LIST";
	case T_PRIMARY_ZONE : return "T_PRIMARY_ZONE";
	case T_READ_ONLY : return "T_READ_ONLY";
	case T_REWRITE_MERGE_VERSION : return "T_REWRITE_MERGE_VERSION";
	case T_STORAGE_FORMAT_VERSION : return "T_STORAGE_FORMAT_VERSION";
	case T_STORAGE_FORMAT_WORK_VERSION : return "T_STORAGE_FORMAT_WORK_VERSION";
	case T_ENGINE : return "T_ENGINE";
	case T_ON : return "T_ON";
	case T_OFF : return "T_OFF";
	case T_CREATE_RESOURCE_UNIT : return "T_CREATE_RESOURCE_UNIT";
	case T_ALTER_RESOURCE_UNIT : return "T_ALTER_RESOURCE_UNIT";
	case T_DROP_RESOURCE_UNIT : return "T_DROP_RESOURCE_UNIT";
	case T_RESOURCE_POOL_OPTION_LIST : return "T_RESOURCE_POOL_OPTION_LIST";
	case T_UNIT : return "T_UNIT";
	case T_UNIT_NUM : return "T_UNIT_NUM";
	case T_UNIT_ID_LIST : return "T_UNIT_ID_LIST";
	case T_CREATE_RESOURCE_POOL : return "T_CREATE_RESOURCE_POOL";
	case T_DROP_RESOURCE_POOL : return "T_DROP_RESOURCE_POOL";
	case T_ALTER_RESOURCE_POOL : return "T_ALTER_RESOURCE_POOL";
	case T_SPLIT_RESOURCE_POOL : return "T_SPLIT_RESOURCE_POOL";
	case T_RESOURCE_UNIT_OPTION_LIST : return "T_RESOURCE_UNIT_OPTION_LIST";
	case T_MAX_CPU : return "T_MAX_CPU";
	case T_MAX_MEMORY : return "T_MAX_MEMORY";
	case T_MAX_IOPS : return "T_MAX_IOPS";
	case T_MAX_DISK_SIZE : return "T_MAX_DISK_SIZE";
	case T_MAX_SESSION_NUM : return "T_MAX_SESSION_NUM";
	case T_MIN_CPU : return "T_MIN_CPU";
	case T_MIN_IOPS : return "T_MIN_IOPS";
	case T_MIN_MEMORY : return "T_MIN_MEMORY";
	case T_BASIC : return "T_BASIC";
	case T_OUTLINE : return "T_OUTLINE";
	case T_EXTENDED : return "T_EXTENDED";
	case T_PARTITIONS : return "T_PARTITIONS";
	case T_TRADITIONAL : return "T_TRADITIONAL";
	case T_JSON : return "T_JSON";
	case T_EXTENDED_NOADDR : return "T_EXTENDED_NOADDR";
	case T_PLANREGRESS : return "T_PLANREGRESS";
	case T_SFU_INT : return "T_SFU_INT";
	case T_SFU_DECIMAL : return "T_SFU_DECIMAL";
	case T_SFU_DOUBLE : return "T_SFU_DOUBLE";
	case T_FOR_UPDATE : return "T_FOR_UPDATE";
	case T_DEFAULT_INT : return "T_DEFAULT_INT";
	case T_DEFAULT_NULL : return "T_DEFAULT_NULL";
	case T_LIMIT_INT : return "T_LIMIT_INT";
	case T_PARTITION_ID_DESC : return "T_PARTITION_ID_DESC";
	case T_PARTITION_ID_SERVER : return "T_PARTITION_ID_SERVER";
	case T_SERVER_TENANT : return "T_SERVER_TENANT";
	case T_ZONE_TENANT : return "T_ZONE_TENANT";
	case T_SWITCH_REPLICA_ROLE : return "T_SWITCH_REPLICA_ROLE";
	case T_SWITCH_RS_ROLE : return "T_SWITCH_RS_ROLE";
	case T_CREATE_TIMESTAMP : return "T_CREATE_TIMESTAMP";
	case T_RECYCLE_REPLICA : return "T_RECYCLE_REPLICA";
	case T_MERGE_CONTROL : return "T_MERGE_CONTROL";
	case T_TENANT_NAME : return "T_TENANT_NAME";
	case T_CACHE_NAME : return "T_CACHE_NAME";
	case T_FILE_ID : return "T_FILE_ID";
	case T_TENANT_LIST : return "T_TENANT_LIST";
	case T_CHANGE_TENANT : return "T_CHANGE_TENANT";
	case T_SET_NAMES : return "T_SET_NAMES";
	case T_SET_CHARSET : return "T_SET_CHARSET";
	case T_CLEAR_LOCATION_CACHE : return "T_CLEAR_LOCATION_CACHE";
	case T_RELOAD_UNIT : return "T_RELOAD_UNIT";
	case T_EMPTY : return "T_EMPTY";
	case T_CONCAT_STRING : return "T_CONCAT_STRING";
	case T_FOUND_ROWS : return "T_FOUND_ROWS";
	case T_BINARY : return "T_BINARY";
	case T_TRANSACTION : return "T_TRANSACTION";
	case T_TRANSACTION_CHARACTERISTICS : return "T_TRANSACTION_CHARACTERISTICS";
	case T_RELOAD_SERVER : return "T_RELOAD_SERVER";
	case T_RELOAD_ZONE : return "T_RELOAD_ZONE";
	case T_MIGRATE_UNIT : return "T_MIGRATE_UNIT";
	case T_CLEAR_MERGE_ERROR : return "T_CLEAR_MERGE_ERROR";
	case T_QEURY_EXPRESSION_LIST : return "T_QEURY_EXPRESSION_LIST";
	case T_UPGRADE_VIRTUAL_SCHEMA : return "T_UPGRADE_VIRTUAL_SCHEMA";
	case T_RUN_JOB : return "T_RUN_JOB";
	case T_EMPTY_QUERY : return "T_EMPTY_QUERY";
	case T_CREATE_OUTLINE : return "T_CREATE_OUTLINE";
	case T_ALTER_OUTLINE : return "T_ALTER_OUTLINE";
	case T_DROP_OUTLINE : return "T_DROP_OUTLINE";
	case T_ADMIN_UPGRADE_CMD : return "T_ADMIN_UPGRADE_CMD";
	case T_GATHER_PLAN_STATISTICS : return "T_GATHER_PLAN_STATISTICS";
	case T_TP_NAME : return "T_TP_NAME";
	case T_TP_NO : return "T_TP_NO";
	case T_ACTION : return "T_ACTION";
	case T_ERROR : return "T_ERROR";
	case T_ERROR_CODE : return "T_ERROR_CODE";
	case T_SYTEM_SETTP_LIST : return "T_SYTEM_SETTP_LIST";
	case T_ALTER_SYSTEM_SETTP : return "T_ALTER_SYSTEM_SETTP";
	case T_ALTER_SYSTEM_DROP_TEMP_TABLE : return "T_ALTER_SYSTEM_DROP_TEMP_TABLE";
	case T_ALTER_SYSTEM_REFRESH_TEMP_TABLE : return "T_ALTER_SYSTEM_REFRESH_TEMP_TABLE";
	case T_ALTER_DISKGROUP_ADD_DISK : return "T_ALTER_DISKGROUP_ADD_DISK";
	case T_ALTER_DISKGROUP_DROP_DISK : return "T_ALTER_DISKGROUP_DROP_DISK";
	case T_OCCUR : return "T_OCCUR";
	case T_TRIGGER_MODE : return "T_TRIGGER_MODE";
	case T_REFRESH_TIME_ZONE_INFO : return "T_REFRESH_TIME_ZONE_INFO";
	case T_REGION : return "T_REGION";
	case T_SET_MERGE_LIST : return "T_SET_MERGE_LIST";
	case T_INTO_VARS_LIST : return "T_INTO_VARS_LIST";
	case T_INTO_FIELD_LIST : return "T_INTO_FIELD_LIST";
	case T_INTO_LINE_LIST : return "T_INTO_LINE_LIST";
	case T_FIELD_TERMINATED_STR : return "T_FIELD_TERMINATED_STR";
	case T_OPTIONALLY_CLOSED_STR : return "T_OPTIONALLY_CLOSED_STR";
	case T_CLOSED_STR : return "T_CLOSED_STR";
	case T_LINE_TERMINATED_STR : return "T_LINE_TERMINATED_STR";
	case T_LINE_START_STR : return "T_LINE_START_STR";
	case T_CHANGE_LIST : return "T_CHANGE_LIST";
	case T_CHANGE_REPLICA : return "T_CHANGE_REPLICA";
	case T_REPLICA_TYPE : return "T_REPLICA_TYPE";
	case T_PCTFREE : return "T_PCTFREE";
	case T_SET_DISK_VALID : return "T_SET_DISK_VALID";
	case T_SQL_STATE : return "T_SQL_STATE";
	case T_SQL_EXCEPTION : return "T_SQL_EXCEPTION";
	case T_SQL_WARNING : return "T_SQL_WARNING";
	case T_SQL_NOT_FOUND : return "T_SQL_NOT_FOUND";
	case T_SP_EXCEPTION_OTHERS : return "T_SP_EXCEPTION_OTHERS";
	case T_SP_PARAM : return "T_SP_PARAM";
	case T_SP_PARAM_LIST : return "T_SP_PARAM_LIST";
	case T_SP_NAME : return "T_SP_NAME";
	case T_SP_ACCESS_NAME : return "T_SP_ACCESS_NAME";
	case T_SP_CREATE : return "T_SP_CREATE";
	case T_SP_ALTER : return "T_SP_ALTER";
	case T_SP_COMPILE_CLAUSE : return "T_SP_COMPILE_CLAUSE";
	case T_SP_EDITIONABLE_CLAUSE : return "T_SP_EDITIONABLE_CLAUSE";
	case T_SP_DROP : return "T_SP_DROP";
	case T_SP_SOURCE : return "T_SP_SOURCE";
	case T_SP_PROC_STMT_LIST : return "T_SP_PROC_STMT_LIST";
	case T_SQL_STMT : return "T_SQL_STMT";
	case T_SP_USING_LIST : return "T_SP_USING_LIST";
	case T_SP_USING_PARAM : return "T_SP_USING_PARAM";
	case T_SP_EXECUTE_IMMEDIATE : return "T_SP_EXECUTE_IMMEDIATE";
	case T_SP_BLOCK_CONTENT : return "T_SP_BLOCK_CONTENT";
	case T_SP_LABELED_BLOCK : return "T_SP_LABELED_BLOCK";
	case T_SP_DECL_LIST : return "T_SP_DECL_LIST";
	case T_SP_DECL : return "T_SP_DECL";
	case T_SP_DECL_COND : return "T_SP_DECL_COND";
	case T_SP_DECL_HANDLER : return "T_SP_DECL_HANDLER";
	case T_SP_DECL_IDENT_LIST : return "T_SP_DECL_IDENT_LIST";
	case T_SP_DECL_DEFAULT : return "T_SP_DECL_DEFAULT";
	case T_SP_DECL_CURSOR : return "T_SP_DECL_CURSOR";
	case T_SP_EXTEND : return "T_SP_EXTEND";
	case T_SP_DELETE : return "T_SP_DELETE";
	case T_SP_DO : return "T_SP_DO";
	case T_SP_IF : return "T_SP_IF";
	case T_SP_ELSE : return "T_SP_ELSE";
	case T_SP_CASE : return "T_SP_CASE";
	case T_SP_WHEN : return "T_SP_WHEN";
	case T_SP_RETURN : return "T_SP_RETURN";
	case T_SP_GOTO : return "T_SP_GOTO";
	case T_SP_CPARAM : return "T_SP_CPARAM";
	case T_SP_CPARAM_LIST : return "T_SP_CPARAM_LIST";
	case T_SP_CALL_STMT : return "T_SP_CALL_STMT";
	case T_SP_INNER_CALL_STMT : return "T_SP_INNER_CALL_STMT";
	case T_SP_ANONYMOUS_BLOCK : return "T_SP_ANONYMOUS_BLOCK";
	case T_SP_NULL : return "T_SP_NULL";
	case T_SP_PIPELINED : return "T_SP_PIPELINED";
	case T_SP_PIPE_ROW : return "T_SP_PIPE_ROW";
	case T_SP_LOOP : return "T_SP_LOOP";
	case T_SP_WHILE : return "T_SP_WHILE";
	case T_SP_REPEAT : return "T_SP_REPEAT";
	case T_SP_LABELED_CONTROL : return "T_SP_LABELED_CONTROL";
	case T_SP_ITERATE : return "T_SP_ITERATE";
	case T_SP_LEAVE : return "T_SP_LEAVE";
	case T_SP_CONDITION : return "T_SP_CONDITION";
	case T_SP_HCOND_LIST : return "T_SP_HCOND_LIST";
	case T_SF_CREATE : return "T_SF_CREATE";
	case T_SF_ALTER : return "T_SF_ALTER";
	case T_SF_DROP : return "T_SF_DROP";
	case T_SF_SOURCE : return "T_SF_SOURCE";
	case T_SF_AGGREGATE_SOURCE : return "T_SF_AGGREGATE_SOURCE";
	case T_SP_SIGNAL : return "T_SP_SIGNAL";
	case T_SP_RESIGNAL : return "T_SP_RESIGNAL";
	case T_SP_SIGNAL_INFO_LIST : return "T_SP_SIGNAL_INFO_LIST";
	case T_SP_SIGNAL_INFO_ITEM : return "T_SP_SIGNAL_INFO_ITEM";
	case T_SP_REF_CURSOR_TYPE : return "T_SP_REF_CURSOR_TYPE";
	case T_RECORD_MEMBER : return "T_RECORD_MEMBER";
	case T_RECORD_MEMBER_LIST : return "T_RECORD_MEMBER_LIST";
	case T_SP_RECORD_TYPE : return "T_SP_RECORD_TYPE";
	case T_SP_COLLECTION_TYPE : return "T_SP_COLLECTION_TYPE";
	case T_SP_DECL_USER_TYPE : return "T_SP_DECL_USER_TYPE";
	case T_SP_NESTED_TABLE_TYPE : return "T_SP_NESTED_TABLE_TYPE";
	case T_SP_ASSOC_ARRAY_TYPE : return "T_SP_ASSOC_ARRAY_TYPE";
	case T_SP_VARRAY_TYPE : return "T_SP_VARRAY_TYPE";
	case T_SP_PROC_OPEN : return "T_SP_PROC_OPEN";
	case T_SP_PROC_FETCH : return "T_SP_PROC_FETCH";
	case T_SP_PROC_CLOSE : return "T_SP_PROC_CLOSE";
	case T_SP_INTO_LIST : return "T_SP_INTO_LIST";
	case T_SP_OBJ_ACCESS_REF : return "T_SP_OBJ_ACCESS_REF";
	case T_SP_INVOKE : return "T_SP_INVOKE";
	case T_SP_ACCESSOR : return "T_SP_ACCESSOR";
	case T_SP_ACCESSOR_KIND : return "T_SP_ACCESSOR_KIND";
	case T_SP_ACCESSOR_LIST : return "T_SP_ACCESSOR_LIST";
	case T_SP_ACCESSIBLE_BY : return "T_SP_ACCESSIBLE_BY";
	case T_SP_DETERMINISTIC : return "T_SP_DETERMINISTIC";
	case T_SP_PARALLEL_ENABLE : return "T_SP_PARALLEL_ENABLE";
	case T_SP_SOURCE_LIST : return "T_SP_SOURCE_LIST";
	case T_SP_RELIES_ON : return "T_SP_RELIES_ON";
	case T_SP_RESULT_CACHE : return "T_SP_RESULT_CACHE";
	case T_SP_CLAUSE_LIST : return "T_SP_CLAUSE_LIST";
	case T_SP_INTEGER_TYPE : return "T_SP_INTEGER_TYPE";
	case T_SP_FOR_LOOP : return "T_SP_FOR_LOOP";
	case T_SP_CURSOR_FOR_LOOP : return "T_SP_CURSOR_FOR_LOOP";
	case T_SP_BOUND_CLAUSE : return "T_SP_BOUND_CLAUSE";
	case T_SP_INDICES_OF_CALUSE : return "T_SP_INDICES_OF_CALUSE";
	case T_SP_VALUES_OF_CALUSE : return "T_SP_VALUES_OF_CALUSE";
	case T_SP_FORALL : return "T_SP_FORALL";
	case T_SP_TYPE : return "T_SP_TYPE";
	case T_SP_ROWTYPE : return "T_SP_ROWTYPE";
	case T_SP_DECL_USER_SUBTYPE : return "T_SP_DECL_USER_SUBTYPE";
	case T_SP_USER_SUBTYPE_RANGE : return "T_SP_USER_SUBTYPE_RANGE";
	case T_SP_USER_SUBTYPE_BASETYPE : return "T_SP_USER_SUBTYPE_BASETYPE";
	case T_SP_PRAGMA_INLINE : return "T_SP_PRAGMA_INLINE";
	case T_SP_PRAGMA_UDF : return "T_SP_PRAGMA_UDF";
	case T_SP_PRAGMA_SERIALLY_REUSABLE : return "T_SP_PRAGMA_SERIALLY_REUSABLE";
	case T_SP_PRAGMA_RESTRICT_REFERENCE : return "T_SP_PRAGMA_RESTRICT_REFERENCE";
	case T_SP_PRAGMA_AUTONOMOUS_TRANSACTION : return "T_SP_PRAGMA_AUTONOMOUS_TRANSACTION";
	case T_SP_PRAGMA_INTERFACE : return "T_SP_PRAGMA_INTERFACE";
	case T_SP_CURSOR_ISOPEN : return "T_SP_CURSOR_ISOPEN";
	case T_SP_CURSOR_FOUND : return "T_SP_CURSOR_FOUND";
	case T_SP_CURSOR_NOTFOUND : return "T_SP_CURSOR_NOTFOUND";
	case T_SP_CURSOR_ROWCOUNT : return "T_SP_CURSOR_ROWCOUNT";
	case T_SP_CURSOR_ROWID : return "T_SP_CURSOR_ROWID";
	case T_SP_CURSOR_BULK_ROWCOUNT : return "T_SP_CURSOR_BULK_ROWCOUNT";
	case T_SP_CURSOR_BULK_EXCEPTIONS : return "T_SP_CURSOR_BULK_EXCEPTIONS";
	case T_SP_CURSOR_BULK_EXCEPTIONS_COUNT : return "T_SP_CURSOR_BULK_EXCEPTIONS_COUNT";
	case T_SP_IMPLICIT_CURSOR_ATTR : return "T_SP_IMPLICIT_CURSOR_ATTR";
	case T_SP_EXPLICIT_CURSOR_ATTR : return "T_SP_EXPLICIT_CURSOR_ATTR";
	case T_SP_CREATE_TYPE : return "T_SP_CREATE_TYPE";
	case T_SP_DROP_TYPE : return "T_SP_DROP_TYPE";
	case T_SP_OBJECT_DEF : return "T_SP_OBJECT_DEF";
	case T_SP_IS_OBJECT : return "T_SP_IS_OBJECT";
	case T_SP_IS_UNDER_OBJECT : return "T_SP_IS_UNDER_OBJECT";
	case T_SP_SQLJ_OBJ_TYPE : return "T_SP_SQLJ_OBJ_TYPE";
	case T_SP_OBJ_FINAL : return "T_SP_OBJ_FINAL";
	case T_SP_OBJ_INISTANTIABLE : return "T_SP_OBJ_INISTANTIABLE";
	case T_SP_OBJ_FINAL_INST_LIST : return "T_SP_OBJ_FINAL_INST_LIST";
	case T_SP_OBJ_ATTR_LIST : return "T_SP_OBJ_ATTR_LIST";
	case T_SP_OBJ_ATTR_AND_ELEMENT_SPEC : return "T_SP_OBJ_ATTR_AND_ELEMENT_SPEC";
	case T_SP_OBJ_ATTR_SPEC : return "T_SP_OBJ_ATTR_SPEC";
	case T_SP_OBJ_SQLJ_TYPE_ATTR : return "T_SP_OBJ_SQLJ_TYPE_ATTR";
	case T_SP_OBJ_ELEMENT_SPEC : return "T_SP_OBJ_ELEMENT_SPEC";
	case T_SP_OBJ_INHERITANCE : return "T_SP_OBJ_INHERITANCE";
	case T_SP_OBJ_RESTRICT : return "T_SP_OBJ_RESTRICT";
	case T_SP_ASSERT_ITEM : return "T_SP_ASSERT_ITEM";
	case T_SP_ASSERT_ITEM_LIST : return "T_SP_ASSERT_ITEM_LIST";
	case T_SP_OBJ_ELEM_SUBPROG_SPEC : return "T_SP_OBJ_ELEM_SUBPROG_SPEC";
	case T_SP_OBJ_SQLJ_FUNC_DECL : return "T_SP_OBJ_SQLJ_FUNC_DECL";
	case T_SP_OBJ_SQLJ_SIG : return "T_SP_OBJ_SQLJ_SIG";
	case T_SP_OBJ_SELF_RESULT : return "T_SP_OBJ_SELF_RESULT";
	case T_SP_OBJ_VARIABLE_NAME : return "T_SP_OBJ_VARIABLE_NAME";
	case T_SP_OBJ_NAME : return "T_SP_OBJ_NAME";
	case T_SP_OBJ_ELEM_CONSTRUCTOR : return "T_SP_OBJ_ELEM_CONSTRUCTOR";
	case T_SP_OBJ_CONSTRUCT_PARAM : return "T_SP_OBJ_CONSTRUCT_PARAM";
	case T_SP_OBJ_SELF_INOUT : return "T_SP_OBJ_SELF_INOUT";
	case T_SP_OBJ_CONSTR_PARAM : return "T_SP_OBJ_CONSTR_PARAM";
	case T_SP_OBJ_MAP_ORDER : return "T_SP_OBJ_MAP_ORDER";
	case T_SP_OBJ_CONSTR_IMPL : return "T_SP_OBJ_CONSTR_IMPL";
	case T_PACKAGE_CREATE : return "T_PACKAGE_CREATE";
	case T_PACKAGE_CREATE_BODY : return "T_PACKAGE_CREATE_BODY";
	case T_PACKAGE_BLOCK : return "T_PACKAGE_BLOCK";
	case T_PACKAGE_BODY_BLOCK : return "T_PACKAGE_BODY_BLOCK";
	case T_PACKAGE_STMTS : return "T_PACKAGE_STMTS";
	case T_PACKAGE_BODY_STMTS : return "T_PACKAGE_BODY_STMTS";
	case T_PACKAGE_ALTER : return "T_PACKAGE_ALTER";
	case T_PACKAGE_DROP : return "T_PACKAGE_DROP";
	case T_SUB_FUNC_DECL : return "T_SUB_FUNC_DECL";
	case T_SUB_FUNC_DEF : return "T_SUB_FUNC_DEF";
	case T_SUB_PROC_DECL : return "T_SUB_PROC_DECL";
	case T_SUB_PROC_DEF : return "T_SUB_PROC_DEF";
	case T_PACKAGE_ALTER_OPTIONS : return "T_PACKAGE_ALTER_OPTIONS";
	case T_EVENT_JOB_EVERY : return "T_EVENT_JOB_EVERY";
	case T_EVENT_JOB_WITH_ON : return "T_EVENT_JOB_WITH_ON";
	case T_EVENT_JOB_CREATE : return "T_EVENT_JOB_CREATE";
	case T_EVENT_JOB_ALTER : return "T_EVENT_JOB_ALTER";
	case T_EVENT_JOB_DROP : return "T_EVENT_JOB_DROP";
	case T_CLEAR_BALANCE_TASK : return "T_CLEAR_BALANCE_TASK";
	case T_BALANCE_TASK_TYPE : return "T_BALANCE_TASK_TYPE";
	case T_CREATE_SYNONYM : return "T_CREATE_SYNONYM";
	case T_DROP_SYNONYM : return "T_DROP_SYNONYM";
	case T_CREATE_FUNC : return "T_CREATE_FUNC";
	case T_DROP_FUNC : return "T_DROP_FUNC";
	case T_PUBLIC : return "T_PUBLIC";
	case T_DLINK : return "T_DLINK";
	case T_IDC : return "T_IDC";
	case T_STRING_LIST : return "T_STRING_LIST";
	case T_ZONE_TYPE : return "T_ZONE_TYPE";
	case T_ALTER_BASELINE : return "T_ALTER_BASELINE";
	case T_SQL_ID : return "T_SQL_ID";
	case T_BASELINE_ID : return "T_BASELINE_ID";
	case T_LOAD_BASELINE : return "T_LOAD_BASELINE";
	case T_ALTER_TABLEGROUP_ACTION_LIST : return "T_ALTER_TABLEGROUP_ACTION_LIST";
	case T_TABLEGROUP_OPTION_LIST : return "T_TABLEGROUP_OPTION_LIST";
	case T_RETURNING : return "T_RETURNING";
	case T_SAMPLE_SCAN : return "T_SAMPLE_SCAN";
	case T_EXPR_NUM : return "T_EXPR_NUM";
	case T_BASE : return "T_BASE";
	case T_INCR : return "T_INCR";
	case T_ALTER_PARTITION_PARTITIONED : return "T_ALTER_PARTITION_PARTITIONED";
	case T_RANDOM : return "T_RANDOM";
	case T_TABLEGROUP_ID : return "T_TABLEGROUP_ID";
	case T_TABLEGROUP_BINDING : return "T_TABLEGROUP_BINDING";
	case T_MAX_USED_PART_ID : return "T_MAX_USED_PART_ID";
	case T_TENANT_ID : return "T_TENANT_ID";
	case T_LOAD_DATA : return "T_LOAD_DATA";
	case T_LOCAL : return "T_LOCAL";
	case T_ESCAPED_STR : return "T_ESCAPED_STR";
	case T_IGNORE_ROWS : return "T_IGNORE_ROWS";
	case T_REFERENCE_OPTION_LIST : return "T_REFERENCE_OPTION_LIST";
	case T_REFERENCE_OPTION : return "T_REFERENCE_OPTION";
	case T_FOREIGN_KEY : return "T_FOREIGN_KEY";
	case T_RESTRICT : return "T_RESTRICT";
	case T_CASCADE : return "T_CASCADE";
	case T_SET_NULL : return "T_SET_NULL";
	case T_NO_ACTION : return "T_NO_ACTION";
	case T_SET_DEFAULT : return "T_SET_DEFAULT";
	case T_ALTER_FOREIGN_KEY_OPTION : return "T_ALTER_FOREIGN_KEY_OPTION";
	case T_FOREIGN_KEY_DROP : return "T_FOREIGN_KEY_DROP";
	case T_FOREIGN_KEY_MATCH : return "T_FOREIGN_KEY_MATCH";
	case T_SIMPLE : return "T_SIMPLE";
	case T_PARTIAL : return "T_PARTIAL";
	case T_CLUSTER_TYPE : return "T_CLUSTER_TYPE";
	case T_USER_INFO : return "T_USER_INFO";
	case T_NO_USE_PX : return "T_NO_USE_PX";
	case T_CLUSTER_NAME_ID : return "T_CLUSTER_NAME_ID";
	case T_SWTICH_CLUSTER_ROLE : return "T_SWTICH_CLUSTER_ROLE";
	case T_ALTER_PARTITION_SPLIT : return "T_ALTER_PARTITION_SPLIT";
	case T_SPLIT_ACTION : return "T_SPLIT_ACTION";
	case T_ALTER_PARTITION_REORGANIZE : return "T_ALTER_PARTITION_REORGANIZE";
	case T_ALTER_PARTITION_TRUNCATE : return "T_ALTER_PARTITION_TRUNCATE";
	case T_ALTER_SUBPARTITION_ADD : return "T_ALTER_SUBPARTITION_ADD";
	case T_ALTER_SUBPARTITION_DROP : return "T_ALTER_SUBPARTITION_DROP";
	case T_ALTER_SUBPARTITION_TRUNCATE : return "T_ALTER_SUBPARTITION_TRUNCATE";
	case T_ANALYZE : return "T_ANALYZE";
	case T_ANALYZE_STATISTICS : return "T_ANALYZE_STATISTICS";
	case T_ANALYZE_FOR_CLAUSE_LIST : return "T_ANALYZE_FOR_CLAUSE_LIST";
	case T_ANALYZE_TABLE : return "T_ANALYZE_TABLE";
	case T_ANALYZE_ALL_COLUMNS : return "T_ANALYZE_ALL_COLUMNS";
	case T_ANALYZE_COLUMN_LIST : return "T_ANALYZE_COLUMN_LIST";
	case T_ANALYZE_SOME_COLUMNS : return "T_ANALYZE_SOME_COLUMNS";
	case T_ANALYZE_COLUMN_INFO : return "T_ANALYZE_COLUMN_INFO";
	case T_ANALYZE_SAMPLE_INFO : return "T_ANALYZE_SAMPLE_INFO";
	case T_MYSQL_UPDATE_HISTOGRAM : return "T_MYSQL_UPDATE_HISTOGRAM";
	case T_MYSQL_DROP_HISTOGRAM : return "T_MYSQL_DROP_HISTOGRAM";
	case T_ANALYZE_MYSQL_COLUMN_LIST : return "T_ANALYZE_MYSQL_COLUMN_LIST";
	case T_SWITCHOVER : return "T_SWITCHOVER";
	case T_SWITCHOVER_LEADER : return "T_SWITCHOVER_LEADER";
	case T_SWITCHOVER_FOLLOWER : return "T_SWITCHOVER_FOLLOWER";
	case T_ALTER_SYSTEM_SET : return "T_ALTER_SYSTEM_SET";
	case T_ALTER_SYSTEM_SET_PARAMETER : return "T_ALTER_SYSTEM_SET_PARAMETER";
	case T_ALTER_SESSION_SET : return "T_ALTER_SESSION_SET";
	case T_ALTER_SESSION_SET_PARAMETER_LIST : return "T_ALTER_SESSION_SET_PARAMETER_LIST";
	case T_USING_INDEX : return "T_USING_INDEX";
	case T_REFERENCES_CLAUSE : return "T_REFERENCES_CLAUSE";
	case T_DROP_USER_LIST : return "T_DROP_USER_LIST";
	case T_CASCADE_CONSTRAINTS : return "T_CASCADE_CONSTRAINTS";
	case T_PURGE : return "T_PURGE";
	case T_STORE_FORMAT : return "T_STORE_FORMAT";
	case T_SPLIT_RANGE : return "T_SPLIT_RANGE";
	case T_SPLIT_LIST : return "T_SPLIT_LIST";
	case T_OPTIMIZE_TABLE : return "T_OPTIMIZE_TABLE";
	case T_OPTIMIZE_TENANT : return "T_OPTIMIZE_TENANT";
	case T_OPTIMIZE_ALL : return "T_OPTIMIZE_ALL";
	case T_XA_START : return "T_XA_START";
	case T_XA_END : return "T_XA_END";
	case T_XA_PREPARE : return "T_XA_PREPARE";
	case T_XA_COMMIT : return "T_XA_COMMIT";
	case T_XA_ROLLBACK : return "T_XA_ROLLBACK";
	case T_REMOVE_CLUSTER : return "T_REMOVE_CLUSTER";
	case T_ADD_CLUSTER : return "T_ADD_CLUSTER";
	case T_TABLE_FLASHBACK_QUERY_TIMESTAMP : return "T_TABLE_FLASHBACK_QUERY_TIMESTAMP";
	case T_TABLE_FLASHBACK_QUERY_SCN : return "T_TABLE_FLASHBACK_QUERY_SCN";
	case T_TABLE_FLASHBACK_QUERY : return "T_TABLE_FLASHBACK_QUERY";
	case T_FAILOVER_TO_LEADER : return "T_FAILOVER_TO_LEADER";
	case T_ENABLE_ROW_MOVEMENT : return "T_ENABLE_ROW_MOVEMENT";
	case T_ORA_ROWSCN : return "T_ORA_ROWSCN";
	case T_TABLE_MODE : return "T_TABLE_MODE";
	case T_RELOAD_GTS : return "T_RELOAD_GTS";
	case T_ALTER_CONSTRAINT_OPTION : return "T_ALTER_CONSTRAINT_OPTION";
	case T_DROP_CONSTRAINT : return "T_DROP_CONSTRAINT";
	case T_CREATE_SAVEPOINT : return "T_CREATE_SAVEPOINT";
	case T_ROLLBACK_SAVEPOINT : return "T_ROLLBACK_SAVEPOINT";
	case T_RELEASE_SAVEPOINT : return "T_RELEASE_SAVEPOINT";
	case T_CONSTRAINT_STATE : return "T_CONSTRAINT_STATE";
	case T_RELY_CONSTRAINT : return "T_RELY_CONSTRAINT";
	case T_NORELY_CONSTRAINT : return "T_NORELY_CONSTRAINT";
	case T_ENABLE_CONSTRAINT : return "T_ENABLE_CONSTRAINT";
	case T_DISABLE_CONSTRAINT : return "T_DISABLE_CONSTRAINT";
	case T_VALIDATE_CONSTRAINT : return "T_VALIDATE_CONSTRAINT";
	case T_NOVALIDATE_CONSTRAINT : return "T_NOVALIDATE_CONSTRAINT";
	case T_RELATION_FACTOR_IN_USE_JOIN_HINT_LIST : return "T_RELATION_FACTOR_IN_USE_JOIN_HINT_LIST";
	case T_NO_USE_MERGE : return "T_NO_USE_MERGE";
	case T_NO_USE_NL : return "T_NO_USE_NL";
	case T_NO_USE_BNL : return "T_NO_USE_BNL";
	case T_CREATE_ROLE : return "T_CREATE_ROLE";
	case T_DROP_ROLE : return "T_DROP_ROLE";
	case T_ALTER_ROLE : return "T_ALTER_ROLE";
	case T_SET_ROLE : return "T_SET_ROLE";
	case T_SET_ROLE_PASSWORD : return "T_SET_ROLE_PASSWORD";
	case T_WITH_ADMIN_OPTION : return "T_WITH_ADMIN_OPTION";
	case T_SYSTEM_GRANT : return "T_SYSTEM_GRANT";
	case T_GRANT_ROLE : return "T_GRANT_ROLE";
	case T_SYSTEM_REVOKE : return "T_SYSTEM_REVOKE";
	case T_REVOKE_ROLE : return "T_REVOKE_ROLE";
	case T_SHOW_TRIGGERS : return "T_SHOW_TRIGGERS";
	case T_ALTER_USER_PROFILE : return "T_ALTER_USER_PROFILE";
	case T_CREATE_PROFILE : return "T_CREATE_PROFILE";
	case T_ALTER_PROFILE : return "T_ALTER_PROFILE";
	case T_DROP_PROFILE : return "T_DROP_PROFILE";
	case T_USER_PROFILE : return "T_USER_PROFILE";
	case T_PROFILE_PARAM_LIST : return "T_PROFILE_PARAM_LIST";
	case T_PROFILE_PAIR : return "T_PROFILE_PAIR";
	case T_PROFILE_UNLIMITED : return "T_PROFILE_UNLIMITED";
	case T_PROFILE_DEFAULT : return "T_PROFILE_DEFAULT";
	case T_PCTUSED : return "T_PCTUSED";
	case T_INITRANS : return "T_INITRANS";
	case T_MAXTRANS : return "T_MAXTRANS";
	case T_STORAGE_OPTIONS : return "T_STORAGE_OPTIONS";
	case T_STORAGE_INITIAL : return "T_STORAGE_INITIAL";
	case T_STORAGE_NEXT : return "T_STORAGE_NEXT";
	case T_STORAGE_MINEXTENTS : return "T_STORAGE_MINEXTENTS";
	case T_STORAGE_MAXEXTENTS : return "T_STORAGE_MAXEXTENTS";
	case T_SIZE_OPTION : return "T_SIZE_OPTION";
	case T_SIZE_INT : return "T_SIZE_INT";
	case T_SIZE_UNLIMITED : return "T_SIZE_UNLIMITED";
	case T_MERGE_HINT : return "T_MERGE_HINT";
	case T_NO_MERGE_HINT : return "T_NO_MERGE_HINT";
	case T_NO_EXPAND : return "T_NO_EXPAND";
	case T_USE_CONCAT : return "T_USE_CONCAT";
	case T_UNNEST : return "T_UNNEST";
	case T_NO_UNNEST : return "T_NO_UNNEST";
	case T_PLACE_GROUP_BY : return "T_PLACE_GROUP_BY";
	case T_NO_PLACE_GROUP_BY : return "T_NO_PLACE_GROUP_BY";
	case T_OPT_QB_NAME : return "T_OPT_QB_NAME";
	case T_INTERVAL_LITERAL : return "T_INTERVAL_LITERAL";
  //	case T_INTERVAL_YM_LITERAL : return "T_INTERVAL_YM_LITERAL";
  //	case T_INTERVAL_DS_LITERAL : return "T_INTERVAL_DS_LITERAL";
	case T_TG_ALTER : return "T_TG_ALTER";
	case T_TG_CREATE : return "T_TG_CREATE";
	case T_TG_DROP : return "T_TG_DROP";
	case T_TG_SOURCE : return "T_TG_SOURCE";
	case T_TG_SIMPLE_DML : return "T_TG_SIMPLE_DML";
	case T_TG_COMPOUND_DML : return "T_TG_COMPOUND_DML";
	case T_TG_INSTEAD_DML : return "T_TG_INSTEAD_DML";
	case T_TG_SYSTEM : return "T_TG_SYSTEM";
	case T_BEFORE : return "T_BEFORE";
	case T_AFTER : return "T_AFTER";
	case T_INSTEAD : return "T_INSTEAD";
	case T_ENABLE : return "T_ENABLE";
	case T_DISABLE : return "T_DISABLE";
	case T_TP_STATEMENT : return "T_TP_STATEMENT";
	case T_TP_EACH_ROW : return "T_TP_EACH_ROW";
	case T_TG_DML_EVENT_OPTION : return "T_TG_DML_EVENT_OPTION";
	case T_TG_DML_EVENT_LIST : return "T_TG_DML_EVENT_LIST";
	case T_TG_DML_EVENT : return "T_TG_DML_EVENT";
	case T_TG_WHEN_CONDITION : return "T_TG_WHEN_CONDITION";
  //	case T_TG_SIMPLE_BODY : return "T_TG_SIMPLE_BODY";
  //	case T_TG_SIMPLE_DECLARE : return "T_TG_SIMPLE_DECLARE";
  //	case T_TG_EXECUTE_BLOCK : return "T_TG_EXECUTE_BLOCK";
	case T_TG_COMPOUND_BODY : return "T_TG_COMPOUND_BODY";
	case T_TG_TIMPING_POINT_SECTION_LIST : return "T_TG_TIMPING_POINT_SECTION_LIST";
	case T_TG_TIMPING_POINT_SECTION : return "T_TG_TIMPING_POINT_SECTION";
	case T_TG_COLUMN_LIST : return "T_TG_COLUMN_LIST";
	case T_TG_REF_OLD : return "T_TG_REF_OLD";
	case T_TG_REF_NEW : return "T_TG_REF_NEW";
	case T_TG_REF_PARENT : return "T_TG_REF_PARENT";
	case T_TG_REF_LIST : return "T_TG_REF_LIST";
	case T_USE_NL_MATERIALIZATION : return "T_USE_NL_MATERIALIZATION";
	case T_NO_USE_NL_MATERIALIZATION : return "T_NO_USE_NL_MATERIALIZATION";
	case T_FLASHBACK_TABLE_TO_TIMESTAMP : return "T_FLASHBACK_TABLE_TO_TIMESTAMP";
	case T_FLASHBACK_TABLE_TO_SCN : return "T_FLASHBACK_TABLE_TO_SCN";
	case T_RELATION_FACTORS : return "T_RELATION_FACTORS";
	case T_TLS_OPTIONS : return "T_TLS_OPTIONS";
	case T_TLS_NONE : return "T_TLS_NONE";
	case T_TLS_SSL : return "T_TLS_SSL";
	case T_TLS_XFZN : return "T_TLS_XFZN";  // x509
	case T_TLS_SPECIFIED : return "T_TLS_SPECIFIED";
	case T_TLS_CIPHER : return "T_TLS_CIPHER";
	case T_TLS_ISSUER : return "T_TLS_ISSUER";
	case T_TLS_SUBJECT : return "T_TLS_SUBJECT";
	case T_ROLLUP_LIST : return "T_ROLLUP_LIST";
	case T_WITH_ROLLUP_CLAUSE : return "T_WITH_ROLLUP_CLAUSE";
	case T_MODIFY_CONSTRAINT_OPTION : return "T_MODIFY_CONSTRAINT_OPTION";
	case T_ALTER_CLUSTER : return "T_ALTER_CLUSTER";
	case T_CLUSTER_INFO : return "T_CLUSTER_INFO";
	case T_CONVERT_TO_STANDBY : return "T_CONVERT_TO_STANDBY";
	case T_MEMSTORE_PERCENT : return "T_MEMSTORE_PERCENT";
	case T_GRANT_SYS_PRIV_ORACLE : return "T_GRANT_SYS_PRIV_ORACLE";
	case T_ORACLE_SYS_PRIV_TYPE : return "T_ORACLE_SYS_PRIV_TYPE";
	case T_ARCHIVE_LOG : return "T_ARCHIVE_LOG";
	case T_BACKUP_DATABASE : return "T_BACKUP_DATABASE";
	case T_BACKUP_MANAGE : return "T_BACKUP_MANAGE";
	case T_RESTORE_TENANT_2 : return "T_RESTORE_TENANT_2";
	case T_GEN_ROWS : return "T_GEN_ROWS";
	case T_LOAD_BATCH_SIZE : return "T_LOAD_BATCH_SIZE";
	case T_REMOTE_OSS : return "T_REMOTE_OSS";
	case T_PHYSICAL_RESTORE_TENANT : return "T_PHYSICAL_RESTORE_TENANT";
	case T_REVOKE_SYSAUTH : return "T_REVOKE_SYSAUTH";
	case T_PQ_MAP : return "T_PQ_MAP";
	case T_ORA_PRIV_TYPE : return "T_ORA_PRIV_TYPE";
	case T_ENABLE_PARALLEL_DML : return "T_ENABLE_PARALLEL_DML";
	case T_DISABLE_PARALLEL_DML : return "T_DISABLE_PARALLEL_DML";
	case T_NO_PX_JOIN_FILTER : return "T_NO_PX_JOIN_FILTER";
	case T_NO_PRED_DEDUCE : return "T_NO_PRED_DEDUCE";
	case T_TRANS_PARAM : return "T_TRANS_PARAM";
	case T_MERGE_RESOURCE_POOL : return "T_MERGE_RESOURCE_POOL";
	case T_IS_TENANT_SYS_POOL : return "T_IS_TENANT_SYS_POOL";
	case T_DUMP_MEMORY : return "T_DUMP_MEMORY";
	case T_FORCE_REFRESH_LOCATION_CACHE : return "T_FORCE_REFRESH_LOCATION_CACHE";
	case T_PROFILE_VERIFY_FUNCTION_NAME : return "T_PROFILE_VERIFY_FUNCTION_NAME";
	case T_ADMIN_ROLLING_UPGRADE_CMD : return "T_ADMIN_ROLLING_UPGRADE_CMD";
	case T_ALTER_INDEX_OPTION_ORACLE : return "T_ALTER_INDEX_OPTION_ORACLE";
	case T_PIVOT : return "T_PIVOT";  // check
	case T_UNPIVOT : return "T_UNPIVOT";
	case T_PIVOT_AGGR_LIST : return "T_PIVOT_AGGR_LIST";
	case T_PIVOT_AGGR : return "T_PIVOT_AGGR";
	case T_PIVOT_IN_LIST : return "T_PIVOT_IN_LIST";
	case T_PIVOT_IN : return "T_PIVOT_IN";
	case T_UNPIVOT_IN_LIST : return "T_UNPIVOT_IN_LIST";
	case T_UNPIVOT_IN : return "T_UNPIVOT_IN";
	case T_USE_SUBPARTITION : return "T_USE_SUBPARTITION";
	case T_ROOTSERVICE_LIST : return "T_ROOTSERVICE_LIST";
	case T_CLUSTER_OPTIONS : return "T_CLUSTER_OPTIONS";
	case T_SP_CREATE_TYPE_BODY : return "T_SP_CREATE_TYPE_BODY";
	case T_SP_CREATE_TYPE_BODY_SRC : return "T_SP_CREATE_TYPE_BODY_SRC";
	case T_SP_CONTRUCTOR_DEF_IN_TYPE : return "T_SP_CONTRUCTOR_DEF_IN_TYPE";
	case T_SP_OBJECT_BODY_DEF : return "T_SP_OBJECT_BODY_DEF";
	case T_SP_CREATE_TYPE_SRC : return "T_SP_CREATE_TYPE_SRC";
	case T_SP_OBJ_ELEMENT_SPEC_LIST : return "T_SP_OBJ_ELEMENT_SPEC_LIST";
	case T_FETCH_CLAUSE : return "T_FETCH_CLAUSE";       // use to support fetch next rows only
	case T_FETCH_TIES_CLAUSE : return "T_FETCH_TIES_CLAUSE";  // use to support fetch next rows with tie
	case T_CREATE_DBLINK : return "T_CREATE_DBLINK";
	case T_DROP_DBLINK : return "T_DROP_DBLINK";
	case T_ALTER_DBLINK : return "T_ALTER_DBLINK";
	case T_LABEL_LIST : return "T_LABEL_LIST";
	case T_PRIMARY_ROOTSERVICE_LIST : return "T_PRIMARY_ROOTSERVICE_LIST";
	case T_LIMIT_UINT : return "T_LIMIT_UINT";
	case T_ALTER_USER : return "T_ALTER_USER";
	case T_ALTER_USER_DEFAULT_ROLE : return "T_ALTER_USER_DEFAULT_ROLE";
	case T_DEFAULT_ROLE : return "T_DEFAULT_ROLE";
	case T_REVERSE : return "T_REVERSE";
	case T_AUTO : return "T_AUTO";
	case T_AUTO_PARTITION : return "T_AUTO_PARTITION";
	case T_ENABLE_SQL_THROTTLE : return "T_ENABLE_SQL_THROTTLE";
	case T_DISABLE_SQL_THROTTLE : return "T_DISABLE_SQL_THROTTLE";
	case T_RT : return "T_RT";
	case T_CPU : return "T_CPU";
	case T_IO : return "T_IO";
	case T_NETWORK : return "T_NETWORK";
	case T_LOGICAL_READS : return "T_LOGICAL_READS";
	case T_QUEUE_TIME : return "T_QUEUE_TIME";
	case T_SQL_THROTTLE_METRICS : return "T_SQL_THROTTLE_METRICS";
	case T_BACKUP_SET_ENCRYPTION : return "T_BACKUP_SET_ENCRYPTION";
	case T_BACKUP_SET_DECRYPTION : return "T_BACKUP_SET_DECRYPTION";
	case T_GROUPING_SETS_LIST : return "T_GROUPING_SETS_LIST";
	case T_CUBE_LIST : return "T_CUBE_LIST";
	case T_MODIFY_READ_TRANSPORT : return "T_MODIFY_READ_TRANSPORT";
	case T_SET_PROTECT_MODE : return "T_SET_PROTECT_MODE";
	case T_INTACT_FAILOVER : return "T_INTACT_FAILOVER";
	case T_RECOVERY_CONTROL : return "T_RECOVERY_CONTROL";
	case T_INDEX_ALTER_PARALLEL : return "T_INDEX_ALTER_PARALLEL";
	case T_CREATE_RESTORE_POINT : return "T_CREATE_RESTORE_POINT";
	case T_DROP_RESTORE_POINT : return "T_DROP_RESTORE_POINT";
	case T_ADMIN_RUN_UPGRADE_JOB : return "T_ADMIN_RUN_UPGRADE_JOB";
	case T_ADMIN_STOP_UPGRADE_JOB : return "T_ADMIN_STOP_UPGRADE_JOB";
	case T_CLUSTER_ACTION_VERIFY : return "T_CLUSTER_ACTION_VERIFY";
	case T_ALTER_USER_PRIMARY_ZONE : return "T_ALTER_USER_PRIMARY_ZONE";
	case T_FLUSH_DAG_WARNINGS : return "T_FLUSH_DAG_WARNINGS";
	case T_MULTI_INSERT : return "T_MULTI_INSERT";            // used to support multi table insert
	case T_INSERT_TABLE_LIST : return "T_INSERT_TABLE_LIST";       // used to support multi table insert
	case T_MULTI_TABLE_INSERT : return "T_MULTI_TABLE_INSERT";      // used to support multi table insert
	case T_MULTI_CONDITION_INSERT : return "T_MULTI_CONDITION_INSERT";  // used to support multi table insert
	case T_CONDITIION_INSERT_LIST : return "T_CONDITIION_INSERT_LIST";  // used to support multi table insert
	case T_CONDITION_INSERT : return "T_CONDITION_INSERT";        // used to support multi table insert
	case T_SINGLE_INSERT : return "T_SINGLE_INSERT";           // used to support multi table insert
	case T_HASH_SUBPARTITION_LIST : return "T_HASH_SUBPARTITION_LIST";
	case T_RANGE_SUBPARTITION_LIST : return "T_RANGE_SUBPARTITION_LIST";
	case T_LIST_SUBPARTITION_LIST : return "T_LIST_SUBPARTITION_LIST";
	case T_HASH_PARTITION_LIST : return "T_HASH_PARTITION_LIST";
	case T_RANGE_PARTITION_LIST : return "T_RANGE_PARTITION_LIST";
	case T_LIST_PARTITION_LIST : return "T_LIST_PARTITION_LIST";
	default:return "Unknown";
	}
}
